/* Begin ATest.cpp
called by Aardvark to test stuff
*/

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

// Scenario:
// The example in this program creates a Job class which contains 10
// variables identified via the EJobVars enum list.
// 
// It inherits from bitfields thereby enabling individual variables to be accesses via
// the appropriate EJobVars enum.
// 
// Characteristics of individual variables are contained in an VarCharacteristics
// struct array which define their respective sizes, max value, min value, name. 
// This array is defined and initialized in the Job.h/.cpp file pair
// 
// Having created a job and initialized it with its bitfield based variables,
// weekday is read and modified via variable specific Get/Set functions.
// It is then read and modified via Bitfield enum driven Get/Set functions.

#include "Globals.h"


void ATest() {

	PRINT_FUNCTION_ENTER_NL
	uint8_t valuesArray[] = {
		 1, //uint8_t _jobType,
		1, //uint8_t _repeatInterval,;
		15, //uint8_t _day,
		6, //uint8_t _weekday,
		1, //uint8_t _scheduleType,
		14, //uint8_t _hour,
		0, //uint8_t _fifteenMin,
		0, //boolean _repeatType,
		1 //uint8_t _repeatNumber,
	};
	// Create a job and initialize its variables
	Job job = Job(valuesArray, sizeof(valuesArray)/sizeof(valuesArray[0]));

	job.PrintVariables();

	uint16_t rawDataSize = sizeof(valuesArray); // Storage in BitfieldStorage
	uint16_t compactedDataSize = sizeof(job.GetDataStorageSize()); // Storage in BitfieldStorage
	PRINT_VALUE_EQUAL_NL(job.GetVarCount());
	PRINT_VALUE_EQUAL_NL(rawDataSize);
	PRINT_VALUE_EQUAL_NL(compactedDataSize);

	// Test getting and setting a variable.
	Serial.println(F("Getting weekday."));
	uint8_t weekday = job.Get((uint8_t)EJobVars::weekday);
	PRINT_VALUE_EQUAL_NL(weekday)
	Serial.println(F("Setting weekday to Sunday (value = 6)."));
	weekday = 6;
	job.Set((uint8_t)EJobVars::weekday, weekday);
	weekday = job.Get((uint8_t)EJobVars::weekday);
	PRINT_VALUE_EQUAL_NL(weekday)
	Serial.println(F("PROBLEM above, Set not working correctly."));

	job.PrintVariables();

	PRINT_FUNCTION_EXIT_NL

} // ATest()

// End ATest.cpp
