// Begin Job.cpp

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

/*
The Job class needs to store a sequence of small sized variables. 
It uses the Bitfield class to store these as fields inside of a 
sequence of bytes. Acces to the Job's data is then possible via generalized 
enum driven Get/set functions.
Thje Bitfield class responds to two needs:
	Store data as bitfields to save on RAM space
	Access data seemlessly via generalized enum driven Get/Set functions.
Data is passed to the Bitfield constructor which stores it as bitfields inside a 
byte sequence. The Job class does not store any data. Data values are provided
to the Bitfield instance as an array of data values via the Job constructor.
*/

static VarCharacteristics BitfieldStorage::varCharacteristics[] = {
	// uint8_t size, uint8_t min, max, char* label;
	{SIZE_JOB_jobStatus, 0, 1,"status" },
	{SIZE_JOB_jobType, 0, 4, "jobType" },
	{SIZE_JOB_repeatInterval, 1, 10, "repeatInterval" },
	{SIZE_JOB_day, 1, 31, "day" },
	{SIZE_JOB_weekday, 1, 7, "weekday" },
	{SIZE_JOB_scheduleType, 1, 4, "scheduleType" },
	{SIZE_JOB_hour, 0, 23, "hour" },
	{SIZE_JOB_fifteenMin, 0, 3, "fifteenMin" },
	{SIZE_JOB_repeatType, 0, 1, "repeatType" },
	{SIZE_JOB_repeatNumber, 0, 10, "repeatNumber" }
};


Job::Job(uint8_t* _valuesArray, uint8_t _varCount)
	: BitfieldStorage(_valuesArray, _varCount)
{
	// Data is contained in values array
}



// End Job.cpp
