/* ATest.cpp
   called by Aardvark to test stuff
*/

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

// Scenario
// When creating classes which contain many variables, it gets to be a bother to create
// individual functions to handle the data, such as GetName/SetName. If the class has 20
// variables, you would have to create 40 functions. Because of C++'s direct memory addressing
// possibilities, these functions may be replaced by generic functions which take an enum
// identifier to specify which data item to work on. It is no big deal, but it makes 
// programming and maintence simpler once the mechanics have been set up.
// 
// The ClassData Framework mechanics is illustrated via the Contact class. It is a bare
// bones class which contains three data items: civility, name, and address. 
// Phase 1 consists in defining data item identifiers via an enum list: the EContactData enum.
// Phase 2 is to place data items inside the charData struct.
// Phase 3 is creating the function GetVarPointer which takes an enum to return the address of 
// a specific data item. For example, do GetVarPointer(EContactData::name) to get the address
// of 'name'. The result of GetVarPointer is cast according to need (char* in this scenario) because
// GetVarPointer returns a type long vaiable.
//
// The scenario below creates three contacts which it places inside a 'myContacts' array
// (Contact* array). It then prints the contents of the array.
// 
// It would have been preferable to have contact inherit from linked list classes as was
// done with Flower/Garden in the Linked List Framework but this is unnecessary work 
// as the objective is to illustrate concepts underlying the Class Data Framework, namely, 
// that of placing all variables in a struct and accessing them via an offset, defined by an enum,
// from the struct's base address.
//

// constructor

#include "Globals.h"

char* contactDataNames[]{ "civility", "name", "address" };

Contact::Contact(char* _civility, char* _name, char* _address) {
	// Start by allocating space for class data and copying contents
	ALLOCATE_COPY_NEW(charData.civility, _civility);
	ALLOCATE_COPY_NEW(charData.name, _name);
	ALLOCATE_COPY_NEW(charData.address, _address);
}

long Contact::GetVarPointer(EContactData _item) {
	uint16_t* basePointer = (uint16_t*)&charData;
	basePointer += (uint16_t)_item * 1;
	uint16_t* destPointer = (uint16_t*)(*basePointer);
	return destPointer;

}
char* Contact::Get(EContactData _item) {
	char* varPointer = GetVarPointer(_item);
	return  varPointer;
}

bool Contact::Set(char* _data, EContactData _item) {
	char* varPointer = GetVarPointer(_item);
	ALLOCATE_COPY_NEW(varPointer, _data);
} // Employee::Set

void  Contact::PrintContact() {

	for (int i = 0; i < (uint8_t) EContactData::end; i++) {
		Serial.print((char*)GetVarPointer((EContactData)i));
		Serial.print(F(" "));
	}
		Serial.println();
}


void ATest() {
	PRINT_FUNCTION_ENTER_NL;

	Serial.println(F("Create three contacts then create array of contacts with them."));
	Contact * sasha = new Contact("Mr.", "Sasha", "42 Main St.");
	Contact* fred = new Contact("Dr.", "Fred", "43 Friedman St.");
	Contact* olivia = new Contact("Mrs.", "Olivia", "981 Sunset Blvd.");
	Contact* myContacts[]{ sasha, fred, olivia };
	uint16_t noContacts = sizeof(myContacts) / sizeof(myContacts[0]);
	Serial.print(F("Total contact of myContacts array = ")); Serial.println(noContacts);
	for (int i = 0; i < noContacts; i++) {
		Serial.print(F("Contact no. ")); Serial.print(i + 1); Serial.print(F(" : "));
		myContacts[i]->PrintContact();
	}

	Serial.println(F("\nGood programming practices require that one should delete allocated items\nwhen they are no longer needed."));
	for (int i = 0; i < noContacts; i++) {
		delete (myContacts[i]);
	}
	for (int i = 0; i < 20; i++) {
		Serial.print(F("."));
		delay(100);
	}
	Serial.println();
	Serial.println(F("Contacts in the myContacts array have been deleted."));

	PRINT_FUNCTION_EXIT_NL;
} // ATest()

// END FILS ATest.cpp
