// Begin DataGroup.cpp
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

DataGroup::DataGroup(char* _name) {
	// Get data group - verifies whether it already exists in storage.
	// if so, gets data
	PRINT_FUNCTION_ENTER_DESC_NL("Short constructor - use data in EEPROM")
	// Make sure name's length within bounds
	uint8_t _nameLength = strlen(_name);
	if ( _nameLength == 0 ) {
		Serial.println(F("DataGroup constructor ERROR - no name provided"));
		return;
	}
	if ( _nameLength > DATA_GROUP_NAME_LENGTH -1 ) { // -1 to account for end null
		Serial.println(F("DataGroup constructor ERROR - name too long - limit is ")); Serial.print(DATA_GROUP_NAME_LENGTH);
		Serial.println(F(" - name length is ")); Serial.println(_nameLength);
		return;
	} // if ( nameLength > DATA_GROUP_NAME_LENGTH )
	
	// Check whether data group is already in EEPROM. If not
	// signal error  by setting
	char tempName[DATA_GROUP_NAME_LENGTH];
	eeprom_read_block (tempName, 0, _nameLength);
	tempName[_nameLength] = 0; // Add end null
	//PRINT_VALUE_EQUAL_DESC_NL("Name in EEPROM", tempName)
	// Check whether data group is in EEPROM
	if (strcmp (_name, tempName) != 0 ) {
		Serial.println(F("DataGroup constructor ERROR - name provided and EEPROM DataGroup differ")); Serial.print(DATA_GROUP_NAME_LENGTH);
		PRINT_VALUE_EQUAL_DESC_NL("\tName provided  : ", strDGheader.name)
		PRINT_VALUE_EQUAL_DESC_NL("\tDataGroup name : ", tempName)
		strcpy(strDGheader.name, "WRONG");
		return;
	} // if (strcmp (_name, tempName) != 0 )
	
	HeaderDataRead();
	//HeaderDataPrint("Constructor END");
	PRINT_FUNCTION_EXIT_NL
	return;
} // DataGroup(char* _name)

DataGroup::DataGroup(uint16_t _dataGroupSize, char* _name, char* _desc, EDataType _dataType, ERecordLength _dataLength, ERecordAcccess _dataAccess)
// Problem doing line initialization on struct component
//: strDGheader.dataType (_dataType), strDGheader.dataLength (_dataLength), strDGheader.dataAccess (_dataAccess)
{
	// Creates new DataGroup - registers it in EEPROM without data
	//PRINT_FUNCTION_ENTER_DESC_NL("Long constructor - new data group needs registering");
	//PRINT_VALUE_EQUAL_NL(_name)
	//PRINT_VALUE_EQUAL_NL(_desc)
	strDGheader.dataType = _dataType;
	strDGheader.dataLength = _dataLength;
	strDGheader.dataAccess = _dataAccess;
	// Currently only char data variable length supported
	if (_dataType != EDataType::charData) {
		Serial.println(F("AddRecords ERROR - data type for char* only - need implement for ints and floats."));
		strDGheader.dataGroupSize = 0;
		PRINT_FUNCTION_EXIT_NL
		return;
	}

	// Make sure name's length within bounds
	uint8_t nameLength = strlen(_name);
	if ( nameLength == 0 ) {
		strcpy(strDGheader.name, "Default DataGroup name");
	}
	if ( nameLength > DATA_GROUP_NAME_LENGTH - 1) {// -1 to account for end null
		Serial.println(F("DataGroup constructor ERROR - name too long - limit is ")); Serial.print(DATA_GROUP_NAME_LENGTH);
		Serial.println(F(" - name length is ")); Serial.println(nameLength);
		return;
	} // if ( nameLength > DATA_GROUP_NAME_LENGTH )
	strcpy (strDGheader.name, _name);
	
	// Make sure description's length within bounds
	uint8_t descLength = strlen(_desc);
	if ( descLength == 0 ) {
		strcpy(strDGheader.desc, "Default DataGroup name");
	}
	if ( descLength > DATA_GROUP_DESCRIPTION_LENGTH - 1) {// -1 to account for end null
		Serial.println(F("DataGroup constructor ERROR - description too long - limit is ")); Serial.print(DATA_GROUP_NAME_LENGTH);
		Serial.println(F(" - description length is ")); Serial.println(_desc);
		return;
	} // if ( nameLength > DATA_GROUP_DESCRIPTION_LENGTH )
	strcpy (strDGheader.desc, _desc);
	
	// Set nextAvailableOffset
	strDGheader.dataGroupSize = _dataGroupSize;
	
	// Register in EEPROM
	//PRINT_VALUE_EQUAL_DESC_NL("DataGroup constructor - registering: ", strDGheader.name);
	HeaderDataWrite();
	//HeaderDataPrint("Constructor END");
	PRINT_FUNCTION_EXIT_NL
} // DataGroup(char* _name)

void DataGroup::HeaderDataWrite() {
	//PRINT_FUNCTION_ENTER_NL
	uint16_t offsetWrite{0};
	// WRITE name
	uint8_t nameLength = strlen(strDGheader.name);
	WRITE_RECORD(strDGheader.name, offsetWrite, nameLength, DATA_GROUP_NAME_LENGTH)
	//eeprom_write_block (name, offsetWrite, nameLength+1);
	//offsetWrite += DATA_GROUP_NAME_LENGTH;
	// WRITE desc
	uint8_t descLength = strlen(strDGheader.desc);
	eeprom_write_block (strDGheader.desc, offsetWrite, descLength);
	offsetWrite += DATA_GROUP_DESCRIPTION_LENGTH;
	//WRITE dataGroupSize
	eeprom_write_block (&(strDGheader.dataGroupSize), offsetWrite, 2);
	offsetWrite += 2;
	// WRITE recordSize
	eeprom_write_block (&(strDGheader.recordSize), offsetWrite, 2);
	offsetWrite += 2;
	// WRITE itemsCount
	eeprom_write_block (&(strDGheader.itemsCount), offsetWrite, 2);
	offsetWrite += 2;
	// WRITE dataType
	eeprom_write_block (&(strDGheader.dataType), offsetWrite, 1);
	offsetWrite += 1;
	// WRITE dataLength
	eeprom_write_block (&(strDGheader.dataLength), offsetWrite, 1);
	offsetWrite += 1;
	// WRITE dataAccess
	eeprom_write_block (&(strDGheader.dataAccess), offsetWrite, 1);
	offsetWrite += 1;
	//PRINT_FUNCTION_EXIT_NL
	return;
} // WriteHeaderData

void DataGroup::HeaderDataRead(){
	//Gets DataGroup info: name, desc, etc.
	//PRINT_FUNCTION_ENTER_NL
	uint16_t offsetRead;
	offsetRead = 0;
	// READ name
	eeprom_read_block (strDGheader.name, offsetRead, DATA_GROUP_NAME_LENGTH+1);
	strDGheader.name[DATA_GROUP_NAME_LENGTH] = 0;
	offsetRead += DATA_GROUP_NAME_LENGTH;
	// READ desc
	eeprom_read_block (strDGheader.desc, offsetRead, DATA_GROUP_DESCRIPTION_LENGTH+1);
	strDGheader.desc[DATA_GROUP_DESCRIPTION_LENGTH]	= 0;
	offsetRead += DATA_GROUP_DESCRIPTION_LENGTH;
	eeprom_read_block (&(strDGheader.dataGroupSize), offsetRead, 2);
	offsetRead += 2;
	// READ recordSize
	eeprom_read_block (&(strDGheader.recordSize), offsetRead, 2);
	offsetRead += 2;
	// READ itemsInDatagroup
	eeprom_read_block (&(strDGheader.itemsCount), offsetRead, 2);
	offsetRead += 2;
	// READ dataType
	eeprom_read_block (&(strDGheader.dataType), offsetRead, 1);
	offsetRead += 1;
	// READ dataLength
	eeprom_read_block (&(strDGheader.dataLength), offsetRead, 1);
	offsetRead += 1;
	// READ dataAccess
	eeprom_read_block (&(strDGheader.dataAccess), offsetRead, 1);
	offsetRead += 1;
	//PRINT_FUNCTION_EXIT_NL
	return;
} // ReadHeaderData

void DataGroup::HeaderDataPrint(char* _desc){
	//Gets DataGroup info: name, desc, etc.
	Serial.print(F("DataGroup header data - ")); Serial.println(_desc);
	//HeaderDataRead();
	PRINT_VALUE_EQUAL_TAB_NL(strDGheader.name)
	PRINT_VALUE_EQUAL_TAB_NL(strDGheader.desc)
	PRINT_VALUE_EQUAL_TAB_NL(sizeof(strDGheader))
	PRINT_VALUE_EQUAL_TAB_NL(strDGheader.dataGroupSize)
	PRINT_VALUE_EQUAL_TAB_NL(strDGheader.recordSize)
	PRINT_VALUE_EQUAL_TAB_NL(strDGheader.itemsCount)
	PRINT_VALUE_EQUAL_TAB_NL((uint8_t) strDGheader.dataType)
	PRINT_VALUE_EQUAL_TAB_NL((uint8_t) strDGheader.dataLength)
	PRINT_VALUE_EQUAL_TAB_NL((uint8_t) strDGheader.dataAccess)
} // PrintStats

bool DataGroup::RecordsAdd(char** _data, uint16_t _arrayCount, EDataType _dataType){
	// Adds records contained in array
	//	If no. of items being added is 2+, adds them at the end
	//  If no. of items being added is 1, tries to insert in a hole bif enough otherwise adds at end

	// char** _data - pointer to data
	// uint16_t _arrayCount - number of items to insert
	// uint16_t _arraysize - size of data to store
	// EDataType _dataType - type of data, one of char, int8, int16, float
	
	PRINT_FUNCTION_ENTER_NL
	
	// Currently only char data variable length supported
	if (_dataType != EDataType::charData) {
		Serial.println(F("AddRecords ERROR - data type for char* only - need implement for ints and floats."));
		PRINT_FUNCTION_EXIT_NL
		return false;
	}

	// Calculate arraySize from _arrayCount
	uint16_t arraySize{0};
	for (int i = 0; i < _arrayCount; i++) {
		arraySize += strlen(_data[i]) + 1;
	}
	PRINT_VALUE_EQUAL_DESC_NL("AddRecords : ", arraySize)

	// Check that there is enough space to store data
	uint16_t remainingSpace = strDGheader.dataGroupSize - GetLastAvailableOffset();
	if (arraySize > remainingSpace ) {
		Serial.println(F("AddRecords ERROR - not enough space to store all data."));
		Serial.print(F("Requested = ")); Serial.print(arraySize);
		Serial.print(F(" - Available = ")); Serial.println(remainingSpace);
		PRINT_FUNCTION_EXIT_NL
		return false;
	}
	bool success{false};
	uint16_t itemSize{0};
	switch (_dataType) {
		case EDataType::charData:
		// size redefined in loop since record lengths differ
		itemSize = 0; // Fake size - gets redefined in write loop
		break;
		case EDataType::int8Data:
		itemSize = 2;
		break;
		case EDataType::int16Data:
		itemSize = 4;
		break;
		case EDataType::floatData:
		itemSize = 4;
		break;
		default:
		Serial.println(F("AddRecords ERROR - Bad dataType - illegal switch exit through default."));
		PRINT_FUNCTION_EXIT_NL
		return false;
		break;
	} // switch

	// variable declarations inside for loop prevents breakpoints from being set
	// Optimization flag used: Optimize debugging experience (-Og)
	// With Optimization set to none - works fine, not need to declare i separately
	//int i;
	//EDataType dataType = _dataType;
	//char* data = _data[i];
	uint16_t noItemsStored{0}; // Running count of items stored
	uint16_t dataItemlength{0};
	uint16_t writeOffset{sizeof(strDGheader)};
	for ( int i = 0; i < _arrayCount; i++) {
		switch (_dataType) {
			case EDataType::charData:
			dataItemlength = strlen(_data[i]);
			eeprom_write_block( &dataItemlength, writeOffset, 2);
			writeOffset += 2;
			eeprom_write_block( _data[i], writeOffset, dataItemlength);
			writeOffset += dataItemlength;
			break;
			case EDataType::int8Data:
			case EDataType::int16Data:
			case EDataType::floatData:
			break;
			Serial.println(F("AddRecords ERROR - data type for char* only - need to program for ints and floats."));
			default:
			Serial.println(F("AddRecords ERROR - Bad dataType - illegal switch exit through default."));
			PRINT_FUNCTION_EXIT_NL
			return false;
			break;
		} // switch
		noItemsStored++;
	} // for ( int i = 0; i < _dataCount; i++)
	strDGheader.itemsCount += noItemsStored;
	uint16_t itemsCountPosition{
		DATA_GROUP_NAME_LENGTH +
		DATA_GROUP_DESCRIPTION_LENGTH +
		2 + // dataGroupSize
		2 // recordSize
	};
	eeprom_write_block(&(strDGheader.itemsCount), itemsCountPosition, 2);
	PRINT_FUNCTION_EXIT_NL
	return true;
} // DataGroup::AddRecords

bool DataGroup::RecordInsert( uint16_t _startOffset, char* _dataToInsert ) {
	PRINT_FUNCTION_ENTER_NL
	//Finds first empty slot as of _startOffset
	bool success{true};
	uint16_t count{0}; // Running count to exit while loop
	uint16_t readOffset{sizeof(strDGheader)}; // Running count to exit while loop
	uint16_t recordLength{0};
	//char* recordData{NULL};
	uint16_t sizeRequested{strlen(_dataToInsert)};
	uint8_t firstByteValue{0};
	int16_t delta{0};
	bool insertionSuccess{false};
	while ( true ) {
		// PRINT_VALUE_EQUAL_DESC_NL( "while ( true ) : ", count)
		// PRINT_VALUE_EQUAL_DESC_NL( "while ( true ) : ", strDGheader.itemsCount)
		if ( count >= strDGheader.itemsCount ) {
			break; // Don't put in while cause debugging bypasses it
		} // if ( count < itemsCount )
		// Get record length
		eeprom_read_block(&recordLength, readOffset, 2);
		readOffset += 2;
		firstByteValue = eeprom_read_byte(readOffset);
		count++;
		delta = recordLength - sizeRequested;
		// Two situations - available slot just right slot size == 0 or
		// slot size >= 3 to record recordLength and minimum of 1 byte of data
		if ( firstByteValue == 0 && (delta == 0  || delta >= 3 ) ) 	{
			// OK found empty slot and is big enough
			uint16_t nullValue{0};
			// Roll back to record length
			readOffset -= 2; // Roll back to beginning of record - set to 0
			eeprom_write_block(&sizeRequested, readOffset, 2);
			readOffset += 2;
			eeprom_write_block(_dataToInsert, readOffset, sizeRequested);
			readOffset += sizeRequested;
			// Now validate remaining space
			if ( delta >= 3) {
				// Set record length and first data byte to 0
				//readOffset += sizeRequested;
				uint16_t newSize = delta - 2;
				eeprom_write_block(&newSize, readOffset, 2);
				readOffset += 2;
				eeprom_write_byte(readOffset, 0);
				// Just created small empty slot, increment number of items in data group
			}
			//PRINT_VALUE_EQUAL_DESC_NL("InsertRecord - inserted: ", _dataToInsert);
			PRINT_FUNCTION_EXIT_NL
			return true;
		} // if ( firstByteValue == 0 && (delta == 0  || delta >= 3 ) )
		readOffset += recordLength; // Go to next slot
	} // while ( count < itemsCount )
	
	// If could not insert into empty slot, is there enough room at end to append
	int16_t remainingSpace = GetDataGroupSize() - GetLastAvailableOffset();
	if ( remainingSpace < sizeRequested ) {
		// Not enough space to store one more record
		PRINT_VALUE_EQUAL_DESC_NL("InsertRecord ERROR - Not enough space for record --> remaining = ", remainingSpace);
		PRINT_VALUE_EQUAL_DESC_NL("\trequested = ", sizeRequested);
		PRINT_FUNCTION_EXIT_NL
		return false;
	}
	
	// Insert at end of data group
	if (insertionSuccess == false) {
		// No slot found, insert at end
		// Insert at end
		readOffset = GetLastAvailableOffset();
		eeprom_write_block(&sizeRequested, readOffset, 2);
		readOffset += 2;
		eeprom_write_block(_dataToInsert, readOffset, sizeRequested);
		readOffset += sizeRequested;
		// Increment items count
		strDGheader.itemsCount++;
		RecordItemsCountInEEPROM();
		readOffset = GetLastAvailableOffset();
		PRINT_FUNCTION_EXIT_NL
		return true;
	} // if ( strncmp( recordData, _data, recordLength) == 0 )
	
	// ,
	PRINT_VALUE_EQUAL_NL("InsertRecord ERROR - Should not exit through bottom.");
	PRINT_VALUE_EQUAL_NL("Record should have been inserted or error declared.");
	PRINT_FUNCTION_EXIT_NL
	return false;
} // DataGroup::InsertRecord

bool DataGroup::RecordRemove(char* _data, EDataType _dataType){
	// Cycles from the beginning of data to remove record
	// Returns true if record found and removed, false otherwise
	bool success(true);
	uint16_t count{0}; // Running count to exit while loop
	uint16_t readOffset{sizeof(strDGheader)}; // Running count to exit while loop
	uint16_t recordLength = 0;
	char* recordData{NULL};
	while ( true ) {
	//	PRINT_VALUE_EQUAL_DESC_NL( "while ( true ) : ", count)
	//	PRINT_VALUE_EQUAL_DESC_NL( "while ( true ) : ", strDGheader.itemsCount)
		if ( count >= strDGheader.itemsCount ) {
			break; // Don't put in while cause debugging bypasses it
		} // if ( count < itemsCount )
		// Get record length
		eeprom_read_block(&recordLength, readOffset, 2);
		readOffset += 2;
		count++;
		// Get record data
		recordData = new char[recordLength];
		eeprom_read_block(recordData, readOffset, recordLength);
		readOffset += recordLength;
		if ( strncmp( recordData, _data, recordLength) == 0 ) {
			// OK found record - set first data byte to 0and exit
			//uint16_t nullValue{0};
			readOffset -= recordLength; // Roll back to beginning of record - set to 0
			eeprom_write_byte  (readOffset, 0);
			delete[] recordData;
			return true;
		} // if ( strncmp( recordData, _data, recordLength) == 0 )
		delete[] recordData;
	} // while ( count < itemsCount )
	
	// Did not find record
	return false;
} // DataGroup::RemoveRecord

// Record functions

void DataGroup::RecordItemsCountInEEPROM() {
	// Records new itemsCount into data base
	eeprom_write_block(strDGheader.itemsCount, 124, 2);
} // DataGroup::RecordItemsCount

uint16_t DataGroup::GetLastAvailableOffset() {
	// Gets offset after last record
	uint16_t count{ 0 };
	uint16_t recordLength{ 0 };
	uint16_t readOffset{ sizeof(strDGheader) };
	uint16_t noOfItemsInDataGroup{strDGheader.itemsCount};
	while ( count < noOfItemsInDataGroup) {
		count++;
		eeprom_read_block (&recordLength, readOffset, 2);
		readOffset += (2 + recordLength);
	} //
	uint16_t lastAvailableOffset = readOffset;
	return lastAvailableOffset;
} // DataGroup::GetLastAvailableOffset


bool DataGroup::Defragment() {
	// Compacts data group to eliminate empty slots, aka remove holes
	// Gets offset after last record
	// Currently bugs after removing first hole, consequently
	// return true if removed first hole after 
	// If no holes removed, returns false
	bool foundHole{false};
	uint16_t count{1}; // One based, first item is 1
	uint16_t recordLength{0};
	uint16_t readOffset{sizeof(strDGheader)};
	uint16_t oldReadOffset{readOffset};
	uint16_t writeOffset{0};
	uint8_t firstByte{0};
	uint16_t lastAvailableOffset = GetLastAvailableOffset();
	uint16_t blockToMoveLength{0};
	uint8_t* dataToMove{0};
	//uint16_t dummyA{10};
	//uint16_t dummyB{20};
	//uint16_t dummyC{0};
	// Auxiliary debugging variables
	uint16_t tempItemsCount = strDGheader.itemsCount;
	
	while ( count <= strDGheader.itemsCount) {
		// Avoid runaway situation
		if (strDGheader.itemsCount > MAX_COUNT) {
			PRINT_VALUE_EQUAL_DESC("Defragment ERROR - no items TOO large - ", strDGheader.itemsCount)
				PRINT_VALUE_EQUAL_TAB_NL(count)
				PRINT_VALUE_EQUAL_TAB_NL(recordLength)
				PRINT_VALUE_EQUAL_TAB_NL(readOffset)
		}
		readOffset = oldReadOffset;
		eeprom_read_block (&recordLength, (void*)readOffset, 2);
		readOffset += 2;
		firstByte = eeprom_read_byte ((void*)readOffset);
		readOffset += recordLength;
		if ( firstByte == 0 ) {
			// Found hole, need to remove record length = recordLength
			// Move everything from here on to end
			// Adjust itemsCount
			foundHole = true;
			//Serial.println("Defragment - moving record");
				//PRINT_VALUE_EQUAL_TAB_NL(count)
			tempItemsCount = strDGheader.itemsCount;
			lastAvailableOffset = GetLastAvailableOffset();
			blockToMoveLength = lastAvailableOffset - readOffset;
			//dummyC = dummyA + dummyB;
			//dataToMove = malloc(blockToMoveLength); // Allocate memory
			dataToMove = new uint8_t[blockToMoveLength]; // Allocate memory
			//if (dataToMove == 0) {
				//// Gets here on second pass despite dataToMove not being zero ???
				//Serial.println("Defragment ERROR - could not allocate memory requested");
					//PRINT_VALUE_EQUAL_TAB_NL(blockToMoveLength)
					//PRINT_VALUE_EQUAL_TAB_NL(count)
					//PRINT_VALUE_EQUAL_TAB_NL(lastAvailableOffset)
					//PRINT_VALUE_EQUAL_TAB_NL(readOffset)
					//PRINT_VALUE_EQUAL_TAB_NL(oldReadOffset)
					//PRINT_VALUE_EQUAL_TAB_NL(writeOffset)
					//PRINT_VALUE_EQUAL_TAB_NL(recordLength)
					//PRINT_VALUE_EQUAL_TAB_NL(strDGheader.itemsCount)
				////free(dataToMove);
				//delete[]dataToMove;
				//Serial.println("Defragment ABORTING");
				//return false;
			//}
			// BUGS here - recordLength changed after read???
			eeprom_read_block (dataToMove, (void*)readOffset, blockToMoveLength);
			// Get block of data
			// Write block of data
			writeOffset = readOffset - recordLength - 2;
			oldReadOffset = writeOffset; // Save read offset since want to restart at same point
			eeprom_write_block (dataToMove, (void*)writeOffset, blockToMoveLength);
			// Get length for next record read
			//eeprom_read_block (&recordLength, writeOffset, 2);
			//readOffset = writeOffset;
			// Finished - free memory
			strDGheader.itemsCount--;
			count--; // Removed empty record, need restart from same position
			//free(dataToMove);
			delete[]dataToMove;
			oldReadOffset = writeOffset; // Set offset to block write
			//return true;
			if ( count == (strDGheader.itemsCount - 1) ) {
				break;
			}
		} else {// if ( firstByte == 0 )
			// Nothing done
			oldReadOffset = readOffset;
		}
		//free(dataToMove);
		//delete[]dataToMove;
		count++;
		tempItemsCount = strDGheader.itemsCount;
	} // 	while ( count < itemsCount) -outer while

RecordItemsCountInEEPROM();
	return foundHole;
} // DataGroup::Defragment

///////////////////  TODO
void DataGroup::RecordsPrint(char* _desc) {
	// Prints all records
	PRINT_FUNCTION_ENTER_NL
	uint16_t count{0};
	uint16_t recordLength{0};
	uint16_t noOfItemsInDataGroup{strDGheader.itemsCount};
	uint16_t readOffset{sizeof(strDGheader)};
	uint8_t firstByte{0};
	char* record{0};
	Serial.print(_desc);
	PRINT_VALUE_EQUAL_DESC_NL(" - ", noOfItemsInDataGroup)
	while ( count < noOfItemsInDataGroup) {
		count++;
		eeprom_read_block (&recordLength, readOffset, 2);
		readOffset += 2;
		record = malloc(recordLength+1);
		eeprom_read_block (record, readOffset, recordLength);
		readOffset += recordLength;
		record[recordLength] = 0;
		PRINT_VALUE_EQUAL(count)
		PRINT_VALUE_EQUAL_DESC(" - ", recordLength)
		if ( record[0] !=0 ) {
			PRINT_VALUE_EQUAL_DESC_NL(" : ", record)
			} else {
			Serial.println(F(" EMPTY"));
		}
		free (record);
	} // while
	PRINT_FUNCTION_EXIT_NL
} // DataGroup::RecordsPrint

// TODO
uint16_t DataGroup::GetRecord(uint16_t* _subArrayToFind, uint16_t _subArrayToFindLength) {
	// Returns offset of variable length record which contains subarray defined by parameters
	// Returns 0 if subarray not found
	// Note that offset is offset of record data, not offset of record length which is 2 bytes long
	// So, if first record is the one ofun, the offset will be 2 (3rd byte), not 0

	uint16_t recordFound{ 0 };

	return recordFound;
} // DataGroup::GetRecord

// Begin DataGroup.cpp
