/* ATest.cpp
called by Aardvark to test stuff
*/

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

// Scenario:
// // The scenario is based on a transferring an array of strings. The concatenated
// strings are handed over to a DataPackets instance whose objective is to generate
// an array of packages to be transmitted.
// 
// A private CompareDataPackets methods checks that individual data items in packets
// and input/output data are identical.
// 
// Data being handled is limited to 162 byte chunks. if data were to be longer, 
// it would need to be broken up. // Breaking up data into manageable chunks needs
// to be done manually.
// 
// The DataPackets class converts data into checksum set packets to be sent. 
// Incoming packets are checksum verified and assembled into a continuous data chunk.
// 
// How data is sent or received depends on the method used. In the scenario presented below, 
// data is represented as global values:
//


#include "Globals.h"

// PACKET SEND TEST SCENARIO - Outgoing data as single long string - 130 bytes total
uint8_t* inputData = {
	(uint8_t*) // cast char string to uint8_t
	"First67890123456789012345678901234567890"   // First packet - 40 bytes
	"SecondGHIJKLMNOPQRSTABCDEFGHIJKLMNOPQRSTUV" // Second packet - 42 bytes
	"Third6789012345678901234567890123456789023" // Third packet - 42 bytes
	"Fourth"                                     // Fourth packet - 6 bytes
};

uint8_t outputData[131]; // strlen(inputData) + 1

// PACKET GET TEST SCENARIO - Incoming data as array of strings - 130 bytes total
// This array is for development purposes (GET part)
char* dataToReceive[] = {
	// Make strings one short to account for end null
	// x symbols are place holders for numeric values fille in at the beginning of the ATest function.
	"abcdefFirst67890123456789012345678901234567890Y",   // First packet - 40 bytes
	"abcdSecondGHIJKLMNOPQRSTABCDEFGHIJKLMNOPQRSTUVY", // Second packet - 42 bytes
	"abcdThird6789012345678901234567890123456789023Y", // Third packet - 42 bytes
	"abcdFourthY"                                     // Fourth packet - 6 bytes
};


// SEND puts incoming data is placed into this arrayD. GET uses this array to place data
// in the destination array (receivedData).
// During processing, intermediatePackets will be used as a uint8_t*[] 
uint8_t** intermediatePackets;

uint16_t InitializePacketsToReceive() {

	//Serial.println(F("Raw data before - cast : (char)"));
	//for (int i = 0; i < 4; i++) {
	//	for (int j = 0; j < 10; j++) {
	//		Serial.print((char)dataToReceive[i][j]); Serial.print(F(","));
	//	}
	//	Serial.println();
	//} // for (int i = 0; i < 4; i++)
	//Serial.println();

	uint8_t noGetPackets{ 0 };
	noGetPackets = sizeof(dataToReceive) / sizeof(dataToReceive[0]);
	//PRINT_VALUE_EQUAL_DESC_NL("Packet no data to get - ", noGetPackets);

	uint16_t chunkSizeGet = { 0 };
	for (int i = 0; i < 4; i++) {
		chunkSizeGet += strlen(dataToReceive[i]);
	}
	// Remove overheads
	chunkSizeGet -= (OVERHEAD_FIRST_PACKET + (noGetPackets - 1) * OVERHEAD_NEXT_PACKET);
	// Add ending null 
	chunkSizeGet += noGetPackets;
	//PRINT_VALUE_EQUAL_DESC_NL("Get data size - ", chunkSizeGet);

	// Start code at offset 0
	//PRINT_VALUE_EQUAL_DESC_NL("Address of dataToReceive inside InitializePacketsToReceive - ", (long)dataToReceive);
	dataToReceive[0][0] = dataToReceive[1][0] = dataToReceive[2][0] = dataToReceive[3][0] = DATA_PACKETS_START_CODE;
	// packet no at offset 1
	dataToReceive[0][1] = 1;  dataToReceive[1][1] = 2;  dataToReceive[2][1] = 3;  dataToReceive[3][1] = 4;
	// packet size at offset 2
	dataToReceive[0][2] = 48;  dataToReceive[1][2] = 48;  dataToReceive[2][2] = 48;  dataToReceive[3][2] = 12;
	// DateID at offset 3
	dataToReceive[0][3] = dataToReceive[1][3] = dataToReceive[2][3] = dataToReceive[3][3] = 1;
	// Checksum as previous to last byte obtained from SEND run.
	dataToReceive[0][46] = 10; dataToReceive[1][46] = 182; dataToReceive[2][46] = 56; dataToReceive[3][10] = 118;
	// END code as last byte
	dataToReceive[0][47] = dataToReceive[1][47] = dataToReceive[2][47] = dataToReceive[3][11] = DATA_PACKETS_END_CODE;
	// First packet only - packet count
	dataToReceive[0][4] = 4;
	// First packet only - chunk size
	dataToReceive[0][5] = 130;
	//Serial.println(F("After assigning decimal values - cast : (uint8_t)"));
	//for (int i = 0; i < 4; i++) {
	//	for (int j = 0; j < 4; j++) {
	//		Serial.print((uint8_t)dataToReceive[i][j]); Serial.print(F(","));
	//	}
	//	Serial.println();
	//}
	//Serial.println();

	return chunkSizeGet;
} // InitializePacketsToReceive

void ATest() {

	PRINT_FUNCTION_ENTER_ASTERIX
		//{ // Nameless block SEND
		uint16_t sendChunkSize = { strlen(inputData) };
	//PRINT_VALUE_EQUAL_DESC_NL("Address of dataToReceive before InitializePacketsToReceive - ", (long)dataToReceive);
	//Serial.println(F("BEFORE InitializePacketsToReceive"));
	//PRINT_VALUE_EQUAL_DESC_NL("Address of dataToReceive after InitializePacketsToReceive - ", (long)dataToReceive);
	//PRINT_VALUE_EQUAL_NL(sendChunkSize);
	//PRINT_VALUE_EQUAL_NL((char)dataToReceive[0][0]);
	//PRINT_VALUE_EQUAL_NL((char)dataToReceive[0][1]);
	//PRINT_VALUE_EQUAL_NL((char)dataToReceive[0][2]);
	//PRINT_VALUE_EQUAL_NL((char)dataToReceive[0][3]);

	uint16_t getChunkSize = InitializePacketsToReceive();
	//Serial.println(F("AFTER InitializePacketsToReceive"));
	//PRINT_VALUE_EQUAL_NL((uint8_t)dataToReceive[0][0]);
	//PRINT_VALUE_EQUAL_NL((uint8_t)dataToReceive[0][1]);
	//PRINT_VALUE_EQUAL_NL((uint8_t)dataToReceive[0][2]);
	//PRINT_VALUE_EQUAL_NL((uint8_t)dataToReceive[0][3]);
	//Verify data chunk size are same
	if (sendChunkSize != getChunkSize) {
		// Abort since there is a problem
		Serial.println(F("chunk sizes not the same, aborting!"));
		PRINT_VALUE_EQUAL_TAB_NL(sendChunkSize);
		PRINT_VALUE_EQUAL_TAB_NL(getChunkSize);
		return;
	}

	bool packetsOK{ true };
	for (int i = 0; i < 4; i++) {
		uint8_t packetID = i + 1;
		uint8_t* packetPtr = dataToReceive[i];
		uint8_t packetSize = packetPtr[2];
		PRINT_VALUE_EQUAL_DESC("Packet ID ", packetID);
		bool packetsOK = VerifyParametersCoherenceGlobal(packetPtr, 4, getChunkSize, i + 1, 10, packetSize);
		if (packetsOK == false) {
			PRINT_VALUE_EQUAL_DESC_NL("\tVerifyParametersCoherenceGlobal detected problem on ", packetID);
		}
		else {
			PRINT_VALUE_EQUAL_DESC_NL("\tVerifyParametersCoherenceGlobal OK on ", packetID);
		}
	}
	if (packetsOK == true) {
		Serial.println(F("INPUT packets OK!"));
	}
	else {
		Serial.println(F("INPUT bad, aborting!"));
		return;
	}
	// END defining test packets


	//Create a DataPacket instance
	Serial.println(F("Creating DataPackets::generatePacketsArray instance"));
	DataPackets generatePacketsArray = DataPackets();
	PRINT_VALUE_EQUAL_DESC_NL("\n\GeneratePacketsArray - ENTER - ", sendChunkSize);
	//uint8_t* generatedPackets[DATA_PACKETS_MAX_COUNT];
	bool generatePacketsArraySuccess = generatePacketsArray.GeneratePacketsArray(inputData, sendChunkSize, 1);
	if (generatePacketsArraySuccess == false) {
		Serial.println(F("ATest  - Aborting, GeneratePacketsArray failed."));
		return;
	}
	else {
		Serial.println(F("ATest  - GeneratePacketsArray successful."));
	}
	uint8_t noOfGeneratedPackets = generatePacketsArray.GetNoPacketsCount();
	PRINT_VALUE_EQUAL_DESC_NL("Checking generated packets - ", noOfGeneratedPackets);
	for (int i = 0; i < noOfGeneratedPackets; i++) {
		uint8_t packetID = i + 1;
		uint8_t* packetPtr = generatePacketsArray.GetPacket(packetID);
		uint8_t packetSize = packetPtr[2];
		PRINT_VALUE_EQUAL_DESC("Packet ID ", packetID);
		packetsOK = VerifyParametersCoherenceGlobal(packetPtr, noOfGeneratedPackets, sendChunkSize, i + 1, 10, packetSize);
		if (packetsOK == false) {
			PRINT_VALUE_EQUAL_DESC_NL("\tVerifyParametersCoherenceGlobal detected problem on ", packetID);
		}
		else {
			PRINT_VALUE_EQUAL_DESC_NL("\tVerifyParametersCoherenceGlobal OK on ", packetID);
		}
	}
	Serial.println(F("GeneratePacketsArray - EXIT\n\n"));

	// List input packets
	uint8_t* packetPtr{ 0 };
	for (int i = 0; i < 4; i++) {
		packetPtr = dataToReceive[i];
		//ListPacketContents(packetPtr, i + 1);
	}
	//PRINT_VALUE_EQUAL_NL((uint8_t)dataToReceive[0][0]);
	//PRINT_VALUE_EQUAL_NL((uint8_t)dataToReceive[0][1]);
	//PRINT_VALUE_EQUAL_NL((uint8_t)dataToReceive[0][2]);
	//PRINT_VALUE_EQUAL_NL((uint8_t)dataToReceive[0][3]);



	PRINT_VALUE_EQUAL_DESC_NL("\n\GenerateDataChunk - ENTER - ", getChunkSize);

	bool successGettingPacket{ false };
	uint8_t* destinationArray = new uint8_t[131];
	DataPackets getDatapackets = DataPackets();
	successGettingPacket = getDatapackets.GetDataFromPacketsArray(dataToReceive, destinationArray);
	if (successGettingPacket == true) {
		Serial.println(F("GenerateDataChunk - EXIT"));
	}
	else {
		Serial.println(F("GenerateDataChunk - problems - EXIT"));
	}

	Serial.println(F("Finished processing - printing result"));
	Serial.println((char*)destinationArray);



//*******************************************
	PRINT_VALUE_EQUAL_DESC_NL("\n\GenerateDataChunk - EXIT - ", getChunkSize);

} // ATest()

// END FILE ATest.cpp
