// Begin DataPackets.cpp

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

#define _DEBUG_DATA_PACKETS_ON
#ifdef _DEBUG_DATA_PACKETS_ON
// GLOBAL
#define _DEBUG_LIST_PACKET_CONTENTS

// PUBLIC
//#define _DEBUG_DATA_PACKETS_DESTRUCTOR
//#define _DEBUG_DATA_GENERATE_PACKETS_ARRAY
#define _DEBUG_GET_DATA_FROM_PACKETS_ARRAY
//#define _DEBUG_DATA_PACKETS_GENERATE_DATA_CHUNK
//#define _DEBUG_DATA_PACKETS_CALCULATE_NO_PACKETS_FROM_CHUNK_SIZE

// PRIVATE
//#define _DEBUG_DATA_PACKETS_DEFINE_PARAMETERS
#define _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE
//#define _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE_GLOBAL
//#define _DEBUG_CHECK_PACKET_DATA_VALIFITY
//#define _DEBUG_DATA_PACKETS_GET
//#define _DEBUG_DATA_PACKETS_CHECK_AVAILABLE
//#define _DEBUG_DATA_PACKETS_GET_TWO_BYTE_SEQUENCE
//#define _DEBUG_DATA_PACKETS_GET_BYTE
//#define _DEBUG_DATA_PACKETS_WAIT_FOR_AVAILABLE
//#define _DEBUG_DATA_PACKETS_GET_IS_AWAKE_PACKET
//#define _DEBUG_DATA_PACKETS_SEND_IS_AWAKE_PACKET
//#define _DEBUG_DATA_PACKETS_VERIFY_CHECKSUM
//#define _DEBUG_DATA_PACKETS_CALCULATE_CHECKSUM
//#define _DEBUG_PRINT_DATA_PACKET_CONTENTS
//#define _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE_GLOBAL


#endif // _DEBUG_DATA_PACKETS_ON

//---------------------------------  GLOBAL FUNTIONS  --------------------------

void ListPacketContents(uint8_t* _packetPtr, uint8_t _packetID) {

#ifdef _DEBUG_LIST_PACKET_CONTENTS
	PRINT_FUNCTION_ENTER_NL;
#endif

	uint8_t packetSize{ 0 };
	uint8_t packetID{ 0 };
	uint8_t headerSize{ OVERHEAD_FIRST_PACKET - 2 };
	// Three situations: header, body, trailer
	packetSize = _packetPtr[2];
	if (_packetID > 1) {
		headerSize = OVERHEAD_NEXT_PACKET - 2;
	}
	PRINT_VALUE_EQUAL_DESC_NL("Printing values for ", _packetID);
	for (int i = 0; i < packetSize; i++) {
		if (i < headerSize || i >(packetSize - 2)) {
			Serial.print((uint8_t)_packetPtr[i]); Serial.print(" ");
		}
		else {
			Serial.print((char)_packetPtr[i]); Serial.print(" ");
		}
	} // for (int i = 0; i < _packetCount; i++)
	Serial.println();

#ifdef _DEBUG_LIST_PACKET_CONTENTS
	PRINT_FUNCTION_EXIT_NL
#endif

} // ListPacketContents

bool VerifyParametersCoherenceGlobal(uint8_t* _packetPtr, uint8_t _packetCount, uint8_t _chunkSize, uint8_t _packetID, uint8_t _noItemsToVerify, uint8_t _packetSize) {
	// FUNCTION DESCRIPTION DataPackets::VerifyParametersCoherenceGlobal()
	// FUNCTION DESCRIPTION
	// Verifies parameters in _packet to detect programming errors
	// 0 Start code
	// 1 packe1 no. - 1 to 4
	// 2 packetSize 
	// 3 DataID - user defined (not checked)
	// 4 packetCount (first packet only)
	// 5 chunk size (169 max) (first packet only)
	// 6 begin data
	// packetSize-3 : end data
	// packetSize-2 : Checksum (not checked)
	// packetSize-1 : End code	// ERROR CHECKING
	// Error checks total : 0
	// PARAMETERS
	// Parameters total : 4
	// uint8_t** packetsArray - array of packets
	// uint8_t _noItemsToVerify - limits items to check, useful for partial checks
	// uint8_t* _packet - packet to be checked
	// uint8_t _packetSize - packet size
	// RETURNS
	// Returns : bool - success/failure
	// PROGRAM DOC END DataPackets::VerifyParametersCoherenceGlobal()
	// PROGRAM FLOW START calling function: DataPackets::VerifyParametersCoherenceGlobal()


#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE_GLOBAL
	Serial.println(F("START: VerifyParametersCoherenceGlobal: "));
	//PRINT_VALUE_EQUAL_TAB_NL(_noItemsToVerify);
	//PRINT_VALUE_EQUAL_TAB_NL(_packetSize);
	//for (int i = 0; i < _noItemsToVerify; i++) {
	//	Serial.print(F("\t")); Serial.print(_packetPtr[i]); Serial.print(F(","));
	//}
	//Serial.println();
#endif

	bool packetOK{ true };
	uint8_t processingCount{ 0 };

	// Check for start code
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (_packetPtr[0] != DATA_PACKETS_START_CODE) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE_GLOBAL;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with START code", _packetPtr[0]);
#endif
		packetOK = false;
	}

	// Check for packet number
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (_packetPtr[1] < 1 || _packetPtr[1] > _packetCount) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE_GLOBAL;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with packet no, outside limits", _packetPtr[1]);
#endif
		packetOK = false;
	}

	// Check packet size
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (_packetPtr[2] != _packetSize) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE_GLOBAL;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with packet size", _packetPtr[2]);
#endif
		packetOK = false;
	}

	// Check packet count - packet 1 only
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (_packetPtr[1] == 1 && _packetPtr[4] != _packetCount) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE_GLOBAL;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with packet count", _packetPtr[4]);
		PRINT_VALUE_EQUAL_DESC_NL("Packet count should be ", _packetCount);
#endif
		packetOK = false;
	}

	// Check chunk size - packet 1 only
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (_packetPtr[1] == 1 && _packetPtr[5] != _chunkSize) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE_GLOBAL;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with chunk size", _packetPtr[5]);
#endif
		packetOK = false;
	}

	// Check end code
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (_packetPtr[_packetSize - 1] != DATA_PACKETS_END_CODE) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE_GLOBAL;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with END CODE", _packetPtr[_packetSize - 1]);
#endif
		packetOK = false;
	}

endProcessing:

	if (packetOK == false) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE_GLOBAL;
		Serial.println(F("VerifyParametersCoherence returning false - there were problems."));
#endif
	}

#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE_GLOBAL
	PRINT_FUNCTION_EXIT_NL
#endif
		return packetOK;
	// PROGRAM FLOW END calling function: VerifyParametersCoherenceGlobal()
} // VerifyParametersCoherenceGlobal()




DataPackets::DataPackets() {
	// FUNCTION DESCRIPTION DataPackets::DataPackets()
	// Send specific constructor
	// Sets private members dataChunk and chunkSize
	// Defines no of packets to send
	// ERROR CHECKING
	// Error checks total : 0
	// PARAMETERS
	// Parameters total : 0
	// RETURNS
	// Returns : Constructor - N/A
	// PROGRAM DOC END DataPackets::DataPackets()
	// PROGRAM FLOW START calling function: DataPackets::DataPackets()


#ifdef _DEBUG_DATA_PACKETS_CONSTRUCTOR
	PRINT_FUNCTION_ENTER_NL;
#endif

#ifdef _DEBUG_DATA_PACKETS_CONSTRUCTOR
	PRINT_FUNCTION_EXIT_NL;
#endif
	// PROGRAM FLOW END calling function: DataPackets::DataPackets()
} // DataPackets()

uint8_t DataPackets::GetPacketSizeForPacketID(uint8_t _packetID, uint8_t _totalNoPackets, uint16_t _chunkSize) {
	// Returns bytesInPacket total packet size
	// _packetID - One based
#ifdef _DEBUG_DATA_GET_PACKET_SIZE_FOR_PACKETID
	PRINT_FUNCTION_ENTER_NL;
#endif

	uint8_t bytesInPacket{ 0 };

	// Handle fist packet
	if (_totalNoPackets == 1) {
		if (_chunkSize == DATA_SIZE_FIRST_PACKET) {
			bytesInPacket = DATA_PACKETS_MAX_PACKET_SIZE;
		}
		else {
			bytesInPacket = _chunkSize + OVERHEAD_FIRST_PACKET;
		}
		return bytesInPacket;
	} // if (noPacketsToSendReceive == 1)

	// Handle last packet
	if (_packetID == _totalNoPackets) {
		// Handle last packet
		uint16_t bytesNotYetSent{ 0 };
		bytesNotYetSent = _chunkSize - (DATA_SIZE_FIRST_PACKET + DATA_SIZE_NEXT_PACKETS * (_packetID - 2));
		bytesInPacket = bytesNotYetSent + OVERHEAD_NEXT_PACKET;
		return bytesInPacket;
	} // if (_packetID == _totalNoPackets)

	//Remaining is intermediate packages
	bytesInPacket = DATA_PACKETS_MAX_PACKET_SIZE;

#ifdef _DEBUG_DATA_GET_PACKET_SIZE_FOR_PACKETID
	PRINT_FUNCTION_EXIT_NL;
#endif
	return bytesInPacket;
} // DataPackets::GetPacketSizeForPacketID

void DataPackets::ListPacketData(uint8_t _packetID, uint8_t _packetSize, bool _listAsChar) {
	// _packetID - One based
	uint8_t* packetPtr{ packetsArray[_packetID - 1] };
	uint8_t dataOffsetPacket{ OVERHEAD_FIRST_PACKET - 2 };
	PRINT_VALUE_EQUAL_DESC_NL("ListPacketData - listing data ", _packetID);
	PRINT_VALUE_EQUAL_TAB_NL((long)packetPtr);
	if (_packetID > 1) {
		dataOffsetPacket = OVERHEAD_NEXT_PACKET - 2;
	}
	for (int i = dataOffsetPacket; i < _packetSize - 3; i++) {
		if (_listAsChar == true) {
			PRINT_VALUE_EQUAL_TAB_NL((char)packetPtr[i]);
		}
		else {
			PRINT_VALUE_EQUAL_TAB_NL(packetPtr[i]);
		}
	}
} // DataPackets::ListPacketData

void DataPackets::CheckPacketDataValidity(uint8_t _packetID, uint8_t _startInPacketOffset, uint16_t _startInChunkOffset, uint8_t _compareLength) {
	// _packetID - One based
#ifdef _DEBUG_CHECK_PACKET_DATA_VALIFITY
	PRINT_FUNCTION_ENTER_NL;
	PRINT_VALUE_EQUAL_TAB_NL(_packetID);
	PRINT_VALUE_EQUAL_TAB_NL(_startInPacketOffset);
	PRINT_VALUE_EQUAL_TAB_NL(_startInChunkOffset);
#endif

	uint8_t packetItem{ 0 };
	uint8_t chunkItem{ 0 };
	uint8_t dataOffsetPacket{ 0 };
	uint8_t dataOffsetChunk{ 0 };
	uint8_t* packetPtr{ packetsArray[_packetID - 1] };
#ifdef _DEBUG_CHECK_PACKET_DATA_VALIFITY
	PRINT_VALUE_EQUAL_TAB_NL((long)packetsArray[_packetID - 1]);
	PRINT_VALUE_EQUAL_TAB_NL((long)packetPtr);
#endif
	for (int i = 0; i < _compareLength; i++) {
		dataOffsetPacket = i + _startInPacketOffset;
		dataOffsetChunk = i + _startInChunkOffset;
		packetItem = packetPtr[dataOffsetPacket];
		chunkItem = chunkData[dataOffsetChunk];
		if (packetItem != chunkItem) {
#ifdef _DEBUG_CHECK_PACKET_DATA_VALIFITY
			PRINT_VALUE_EQUAL_DESC_NL("CompareDataPackets - mismatch between data and chunk at ", i);
			PRINT_VALUE_EQUAL_TAB_NL(dataOffsetPacket);
			PRINT_VALUE_EQUAL_TAB_NL((char)packetPtr[dataOffsetPacket]);
			PRINT_VALUE_EQUAL_TAB_NL(dataOffsetChunk);
			PRINT_VALUE_EQUAL_TAB_NL((char)chunkData[dataOffsetChunk]);
#endif
		} // if (packetItem != dataItem)
	} // for (int i = 0; i < _compareLength; i++)

#ifdef _DEBUG_CHECK_PACKET_DATA_VALIFITY
	PRINT_FUNCTION_EXIT_NL
#endif
		return;
} // DataPackets::CompareDataPackets

bool DataPackets::GeneratePacketsArray(uint8_t* _chunkData, uint16_t _chunkSize, uint8_t _chunkID) {
	// FUNCTION DESCRIPTION DataPackets::PacketsSend()
	// FUNCTION DESCRIPTION
	// Simplified version, one packet only - see V11 or earlier for two+ packets data transfer
	// Defines based on dataSize defined by constructor - size of dataPacket including OVERHEAD bytes, end byte, size bytes...
	// _dataID - identifier as to data
	// Each packet has start byte, packetSize, dataSize, packetID, dataID checksum in next to last byte, end byte
	// First packet has additional bytes: total data length to send, data.
	// first packet sends less data than succeeding packets, initial packetID should be 1
	// ERROR CHECKING
	// Error checks total : 0
	// PARAMETERS
	// Parameters total : 3
	// uint8_t* _chunkData - Contains data to send
	// uint8_t* _chunkSize - size of _chunkData
	// uint8_t _dataID - ID of data to send - no currently used for anything, just for future use
	// RETURNS
	// Returns : bool - success/failure
	// PROGRAM DOC END DataPackets::PacketsSend()
	// PROGRAM FLOW START calling function: DataPackets::PacketsSend()

#ifdef _DEBUG_DATA_GENERATE_PACKETS_ARRAY
	PRINT_FUNCTION_ENTER_ASTERIX;
#endif

	bool packetOK{ true }; // Set by VerifyParametersCoherence.

	// Set class variables from parameters
	chunkData = _chunkData;
	chunkSize = _chunkSize;
	chunkID = _chunkID;
	if (chunkData == 0) {
		Serial.println("GeneratePacketsArray - Aborting - Unallocated chunk data provided.");
		return false;
	}
	if (chunkSize == 0) {
		Serial.println("GeneratePacketsArray - Aborting - Zero sized chunk size provided.");
		return false;
	}
	noPacketsToSendReceive = CalculateNoPacketsFromChunkSize(chunkSize);

	// Send packets
#ifdef _DEBUG_DATA_GENERATE_PACKETS_ARRAY
	PRINT_VALUE_EQUAL_DESC_NL("GeneratePacketsArray entering - ", noPacketsToSendReceive);
#endif
	uint8_t startDataInPacketOffset{ 0 };
	uint8_t endDataInPacketOffset{ 0 };
	uint8_t startDataInChunk{ 0 };
	uint8_t currentPacketSize{ 0 };
	uint8_t dataBytesToTransfer{ 0 };
	uint16_t totalBytesSent{ 0 }; // uint16_t just in case want to make it beyond 256
	uint16_t totalDataBytesSent{ 0 };


	//***********************************
	for (int i = 0; i < noPacketsToSendReceive; i++) {
		// define packet size - differentiate first, from intermediate from last
		currentPacketSize = GetPacketSizeForPacketID(i, noPacketsToSendReceive, chunkSize); // One based
#ifdef _DEBUG_DATA_GENERATE_PACKETS_ARRAY
		PRINT_VALUE_EQUAL_DESC_NL("GeneratePacketsArray - handling ", i);
		PRINT_VALUE_EQUAL_DESC_NL("Allocating memory ", currentPacketSize);
		//PRINT_VALUE_EQUAL_TAB_NL(noPacketsToSendReceive)
#endif
// Allocate memory for packet
		packetsArray[i] = new uint8_t[currentPacketSize]; // Shorthand to ease programming
		uint8_t* packetPtr{ packetsArray[i] };
#ifdef _DEBUG_DATA_GENERATE_PACKETS_ARRAY
		PRINT_VALUE_EQUAL_TAB_NL((long)packetsArray[i]);
		PRINT_VALUE_EQUAL_TAB_NL((long)packetPtr);
#endif
		packetPtr[0] = DATA_PACKETS_START_CODE; // start code
		packetPtr[1] = i + 1; // packet no one based
		packetPtr[2] = currentPacketSize; // packet size
		packetPtr[3] = _chunkID; // DataID - not used
		packetPtr[currentPacketSize - 1] = DATA_PACKETS_END_CODE;
		startDataInPacketOffset = 0;
		endDataInPacketOffset = currentPacketSize - 2 - 1;
		// Special case first packet
		if (i == 0) {
			packetPtr[4] = noPacketsToSendReceive;
			packetPtr[5] = chunkSize;
			startDataInPacketOffset = OVERHEAD_FIRST_PACKET - 2;
		}
		else {
			startDataInPacketOffset = OVERHEAD_NEXT_PACKET - 2;
		}
		// Transfer data  dataChunk to packetToSend;
		dataBytesToTransfer = endDataInPacketOffset - startDataInPacketOffset + 1;
		startDataInChunk = totalDataBytesSent;
#ifdef _DEBUG_DATA_GENERATE_PACKETS_ARRAY
		PRINT_VALUE_EQUAL_TAB_NL(startDataInPacketOffset);
		PRINT_VALUE_EQUAL_TAB_NL(endDataInPacketOffset);
		PRINT_VALUE_EQUAL_TAB_NL(startDataInChunk);
		PRINT_VALUE_EQUAL_TAB_NL(currentPacketSize);
		PRINT_VALUE_EQUAL_TAB_NL(dataBytesToTransfer);
		PRINT_VALUE_EQUAL_TAB_NL(totalBytesSent);
		PRINT_VALUE_EQUAL_TAB_NL(totalDataBytesSent);
#endif
		// Transfer chunk data to packet
		uint8_t dataItem{ 0 };
		for (int i = 0; i < dataBytesToTransfer; i++) {
			dataItem = chunkData[i + startDataInChunk];
			packetPtr[i + startDataInPacketOffset] = dataItem;
			//PRINT_VALUE_EQUAL_TAB((char)dataItem); // This reports OK
			//PRINT_VALUE_EQUAL_TAB_NL((char)packetPtr[i + startDataInPacketOffset]); // This reports OK
		} // for (int i = start; i < dataBytesToTransfer; i++)
		totalBytesSent += currentPacketSize;
		totalDataBytesSent += dataBytesToTransfer;
		uint8_t checkSum{ 0 };
		// Exclude starter byte, checksum byte and end byte };
		checkSum = CalculateChecksum(packetPtr, 1, currentPacketSize - 3);
		packetPtr[currentPacketSize - 2] = checkSum;
#ifdef _DEBUG_DATA_GENERATE_PACKETS_ARRAY
		PRINT_VALUE_EQUAL(i);
		PRINT_VALUE_EQUAL_NL(checkSum);
#endif
		// List packet content
		//ListPacketData(i+1, currentPacketSize, true); // True means list char data asd opposed to int
		// This is really checking that programming is OK hence commented out since found OK
		CheckPacketDataValidity(i + 1, startDataInPacketOffset, startDataInChunk, dataBytesToTransfer);
		//			bool checksumOK = VerifyChecksum(packetToSend, packetSize, 1, packetSize - 3);
		//			if (checksumOK == false) {
		//#if defined(ERROR_REPORTING_ON) || defined(_DEBUG_DATA_PACKETS_SEND)
		//				Serial.println("checkSum NOT OK");
		//				PRINT_FUNCTION_EXIT_NL
		//#endif
		//				return false;
		//			}

		// Finished preparations for packet - send it
		packetOK = VerifyParametersCoherence(i + 1, 10, currentPacketSize);
		//delete[] packetPtr;
	} // for (int packetID = 0; packetID < noPacketsToSend; noPacketsToSend++)

#ifdef _DEBUG_DATA_GENERATE_PACKETS_ARRAY
	PRINT_FUNCTION_EXIT_NL
#endif
		if (packetOK == true) {
			return packetsArray;
		}
		else {
			return 0;
		}
	// PROGRAM FLOW END calling function: DataPackets::PacketsSend()
}; // PacketsSend(byte startCode)

bool DataPackets::VerifyParametersCoherence(uint8_t _packetID, uint8_t _noItemsToVerify, uint8_t _packetSize) {
	// FUNCTION DESCRIPTION DataPackets::VerifyParametersCoherence()
	// FUNCTION DESCRIPTION
	// Verifies parameters in _packet to detect programming errors
	// 0 Start code
	// 1 packe1 no. - 1 to 4
	// 2 packetSize 
	// 3 DataID - user defined (not checked)
	// 4 packetCount (first packet only)
	// 5 chunk size (169 max) (first packet only)
	// 6 begin data
	// packetSize-3 : end data
	// packetSize-2 : Checksum (not checked)
	// packetSize-1 : End code	// ERROR CHECKING
	// Error checks total : 0
	// PARAMETERS
	// Parameters total : 3
	// uint8_t _noItemsToVerify - limits items to check, useful for partial checks
	// uint8_t* _packet - packet to be checked
	// uint8_t _packetSize - packet size
	// RETURNS
	// Returns : bool - success/failure
	// PROGRAM DOC END DataPackets::VerifyParametersCoherence()
	// PROGRAM FLOW START calling function: DataPackets::VerifyParametersCoherence()


	uint8_t* packetPtr = packetsArray[_packetID - 1]; // shorthand
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE
	Serial.println(F("START: VerifyParametersCoherence: "));
	PRINT_VALUE_EQUAL_TAB_NL(_noItemsToVerify);
	PRINT_VALUE_EQUAL_TAB_NL(_packetSize);
	for (int i = 0; i < _noItemsToVerify; i++) {
		Serial.print(F("\t")); Serial.print(packetPtr[i]); Serial.print(F(","));
	}
	Serial.println();
#endif

	bool packetOK{ true };
	uint8_t processingCount{ 0 };

	// Check for start code
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (packetPtr[0] != DATA_PACKETS_START_CODE) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with START code", packetPtr[0]);
#endif
		packetOK = false;
	}

	// Check for packet number
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (packetPtr[1] < 1 || packetPtr[1] > noPacketsToSendReceive) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with packet no, outside limits", packetPtr[1]);
#endif
		packetOK = false;
	}

	// Check packet size
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (packetPtr[2] != _packetSize) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with packet size", packetPtr[2]);
#endif
		packetOK = false;
	}

	// Check packet count - packet 1 only
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (packetPtr[1] == 1 && packetPtr[4] != noPacketsToSendReceive) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with packet count", packetPtr[4]);
		PRINT_VALUE_EQUAL_DESC_NL("Packet count should be ", noPacketsToSendReceive);
#endif
		packetOK = false;
	}

	// Check chunk size - packet 1 only
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (packetPtr[1] == 1 && packetPtr[5] != chunkSize) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with chunk size", packetPtr[5]);
#endif
		packetOK = false;
	}

	// Check end code
	processingCount++;
	if (processingCount > _noItemsToVerify) goto endProcessing;
	if (packetPtr[_packetSize - 1] != DATA_PACKETS_END_CODE) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE;
		PRINT_VALUE_EQUAL_DESC_NL("PROBLEM with END CODE", packetPtr[_packetSize - 1]);
#endif
		packetOK = false;
	}

endProcessing:

	if (packetOK == false) {
		Serial.println(F("VerifyParametersCoherence returning false - there were problems."));
	}

#ifdef _DEBUG_DATA_PACKETS_VERIFY_PARAMETERS_COHERENCE
	PRINT_FUNCTION_EXIT_NL
#endif
		return packetOK;
	// PROGRAM FLOW END calling function: DataPackets::VerifyParametersCoherence()
} // VerifyParametersCoherence()

bool DataPackets::GetDataFromPacketsArray(uint8_t** _packetsArray, uint8_t* _destinationArray) {
	// FUNCTION DESCRIPTION DataPackets::PacketsGet()
	// FUNCTION DESCRIPTION
	// Checks packet gets data offset if no problems.
	// Packet supposedly checked by VerifyParametersCoherence
	// ERROR CHECKING
	// Error checks total : 0
	// PARAMETERS
	// Parameters total : 2
	// uint8_t* _packetData - packet data to process
	// uint8_t _packetID - ID of whose data is to be gotten
	// RETURNS
	// Returns : offset of packet data
	// PROGRAM DOC END DataPackets::PacketsGet()
	// PROGRAM FLOW START calling function: DataPackets::PacketsGet()

	// NOTE: all xBee related code has been deleted

#ifdef _DEBUG_GET_DATA_FROM_PACKETS_ARRAY
	PRINT_FUNCTION_ENTER_NL;
#endif

	// Since do not know how to assign and int** to an int*[]; 
	// copy subarray addresses
	noPacketsToSendReceive = _packetsArray[0][4];
	if (noPacketsToSendReceive == 0) {
		Serial.println(F("GetPackets returned noPackets 0 - Aborting"));
		return false;
	}

	for (int i = 0; i < noPacketsToSendReceive; i++) { // Start with first packet then when debugged, do others.
		packetsArray[i] = _packetsArray[i];
	}

	uint8_t* packetPtr{ 0 }; // convenience variable
	// Start by defining chuck size and packet count - contained in dataChunk if dealing with packet 1
	packetPtr = packetsArray[0];
	chunkSize = packetPtr[5];
	if (chunkSize == 0) {
		Serial.println(F("GetPackets returned chunkSize 0 - Aborting"));
		return false;
	}
	uint8_t dataOffsetInPacket{ 0 };
	uint8_t totalDataObtained{ 0 };
	uint8_t packetSize{ packetPtr[2] };
	uint8_t dataOffset{ OVERHEAD_FIRST_PACKET - 2 };
	uint8_t dataSize{ 0 };
	uint8_t packetID{ 0 };
	uint8_t srcOffset{ 0 };
	uint8_t destOffset{ 0 };
	for (int i = 0; i < noPacketsToSendReceive; i++) { // Start with first packet then when debugged, do others.
		packetID = i + 1;
		if (i > 0) {
			dataOffset = OVERHEAD_NEXT_PACKET - 2;
		}
		PRINT_VALUE_EQUAL_DESC_NL("Processing  ", packetID);
		bool successGettingPacket{ false };
		packetPtr = packetsArray[i];
		packetSize = packetPtr[2];
		PRINT_VALUE_EQUAL_TAB_NL(dataOffset);
		PRINT_VALUE_EQUAL_TAB_NL(dataSize);
		PRINT_VALUE_EQUAL_TAB_NL(packetSize);
		bool packetOK{ true };
		//VerifyParametersCoherence(i+1, 10, packetSize);
		if (packetOK == false) {
			PRINT_VALUE_EQUAL_DESC_NL("GenerateDataChunk - Aborting on ", i + 1);
			#ifdef _DEBUG_GET_DATA_FROM_PACKETS_ARRAY
			PRINT_FUNCTION_EXIT_NL;
			#endif
			return false;
		}

		if (i == 0) {
			// OK dealing with packet 1
			dataOffset = OVERHEAD_FIRST_PACKET - 2;
		}
		dataSize = packetSize - dataOffset - 2;

		// Transfer data to destination array
		PRINT_VALUE_EQUAL_TAB_NL(dataOffset);
		PRINT_VALUE_EQUAL_TAB_NL(totalDataObtained);
		for (int k = 0; k < dataSize; k++) {
			srcOffset = k + dataOffset;
			destOffset = k + totalDataObtained;
			_destinationArray[destOffset] = packetPtr[srcOffset];
			Serial.write((char)_destinationArray[destOffset]);
			Serial.write(" ");
		}
		Serial.println();
		totalDataObtained += dataSize;
		_destinationArray[totalDataObtained] = 0;
	} // for (int i = 0; i < 4; i++) 

	//Serial.println(F("Printing _destinationArray"));
	//Serial.println((char*)_destinationArray);
	//for (int i = 0; i < 130; i++) {
	//	Serial.write((char)_destinationArray[i]);
	//	Serial.write(" ");
	//}
	//Serial.println();
#ifdef _DEBUG_GET_DATA_FROM_PACKETS_ARRAY
	PRINT_FUNCTION_EXIT_NL;
#endif

	return true;
} // GetDataFromPacketsArray


//	//********************************************

//	uint8_t startDataInPacketOffset{ OVERHEAD_FIRST_PACKET - 2 };
//	uint8_t endDataInPacketOffset{ chunkSize - 3 };
//	PRINT_VALUE_EQUAL_NL(startDataInPacketOffset);
//	PRINT_VALUE_EQUAL_NL(endDataInPacketOffset);
//	uint8_t startDataInChunk{ 0 };
//	uint8_t endDataInChunk{ 0 };
//	uint16_t totalBytesReceived{ 0 }; // uint16_t just in case want to make it beyond 256
//	endDataInChunk = (chunkSize <= chunkSize ? chunkSize : DATA_SIZE_FIRST_PACKET) - 1;
//	uint8_t receivedDataItem{ 0 };;
//	for (int i = startDataInPacketOffset; i <= endDataInChunk; i++) {
//		uint8_t* receivedDataItem = dataToReceive[0][i + totalBytesReceived];
//		chunkData[i] = receivedDataItem;
//	}
//	totalBytesReceived += chunkSize;
//
//	successGettingPackets = VerifyParametersCoherence(10, chunkData, chunkSize);
//	// Verify checksum
//	uint8_t checkSum{ 0 };
//	uint8_t checksumObtained{ 0 };
//	// Exclude starter byte, checksum byte and end byte };
//	checksumObtained = chunkData[chunkSize - 2];
//	checkSum = CalculateChecksum(chunkData, 1, chunkSize - 3);
//	if (checksumObtained != checkSum) {
//		Serial.println(F("PacketsGet - ERROR - Bad checksum."));
//		return false;
//	}
//
//#ifdef _DEBUG_DATA_PACKETS_GET
//	PRINT_FUNCTION_EXIT_NL
//#endif
//		return successGettingPackets;
//	// PROGRAM FLOW END calling function: GlobalGetInfoFromLittleGuy()
//} // PacketsGet()
//
uint8_t DataPackets::CalculateNoPacketsFromChunkSize(uint16_t _dataSize) {
	// returns number of packets to send, 0 if too large
#ifdef _DEBUG_DATA_PACKETS_CALCULATE_NO_PACKETS_FROM_CHUNK_SIZE
#endif _//DEBUG_DATAGROUP_CALCULATE_NO_PACKETS_FROM_CHUNK_SIZE
	uint8_t totalPackets{ 0 };

	if (_dataSize > DATA_CHUNK_MAX_SIZE) {
		uint8_t tempData{ DATA_CHUNK_MAX_SIZE };
		totalPackets = 0;
		PRINT_VALUE_EQUAL_DESC_NL("CalculateNoPacketsToSend - Aborting, data too large, need to break up data into several chunks : ", chunkSize);
		PRINT_VALUE_EQUAL_NL(tempData);
	}
	else {
		if (_dataSize > DATA_SIZE_FIRST_PACKET) {
			uint8_t dataSizeRemaining = _dataSize - DATA_SIZE_FIRST_PACKET;
			totalPackets = (dataSizeRemaining / DATA_SIZE_NEXT_PACKETS) + 1;
			if (dataSizeRemaining % DATA_SIZE_NEXT_PACKETS > 0) totalPackets++;
		}
#ifdef _DEBUG_DATA_PACKETS_CALCULATE_NO_PACKETS_FROM_CHUNK_SIZE
		PRINT_VALUE_EQUAL_DESC_NL("CalculateNoPacketsToSend - One or more chunks required to send data : ", chunkSize);
		PRINT_VALUE_EQUAL_DESC_NL("Data will be sent as first package + additional packages ", totalPackets);
#endif _//DEBUG_DATAGROUP_CALCULATE_NO_PACKETS_FROM_CHUNK_SIZE
	} // if (dataToSendSize > DATA_CHUNK_MAX_SIZE)
	return totalPackets;
}


//-------------------------------------------- CalculateChecksum() -----------------------------------
uint8_t DataPackets::CalculateChecksum(byte* _dataBuffer, uint8_t _start, uint8_t _end) {
	// FUNCTION DESCRIPTION DataPackets::CalculateChecksum()
	// FUNCTION DESCRIPTION
	// Calculates checksum: the addition of all values between i = 3 and i = length - 2
	// it  excludes the starter byte, checksum byte and end bytes
	// it starts at index i =  3 and stops at index i = lengthFrame - 2
	// ERROR CHECKING
	// Error checks total : 0
	// PARAMETERS
	// Parameters total : 3
	// byte* _dataBuffer - Data in which checksum is to be calculated
	// uint8_t _start - offset start into _dataBuffer
	// uint8_t _end - offset end into _dataBuffer
	// RETURNS
	// Returns : bool - success/failure
	// PROGRAM DOC END DataPackets::CalculateChecksum()

#ifdef _DEBUG_DATA_PACKETS_CALCULATE_CHECKSUM
	PRINT_FUNCTION_ENTER_NL;
#endif
	uint8_t dataForChecksum{ 0 }; // initialize just in case
	uint8_t dataPacketCheckSum{ 0 }; // initialize just in case

	for (uint8_t i = _start; i <= _end; i++) {
		dataForChecksum += _dataBuffer[i];
	}

	dataPacketCheckSum = 0xFF - (dataForChecksum & 0xFF);

#ifdef _DEBUG_DATA_PACKETS_CALCULATE_CHECKSUM
	PRINT_FUNCTION_EXIT_NL;
#endif
	return dataPacketCheckSum;
} // CalculateChecksum()

//-------------------------------------------- VerifyChecksum() -----------------------------------
bool DataPackets::VerifyChecksum(uint8_t* _packet, uint8_t _packetSize, uint8_t _start, uint8_t _end) {
	// FUNCTION DESCRIPTION DataPackets::VerifyChecksum()
	// FUNCTION DESCRIPTION
	// Checksum is the addition of all values between i = _start and i = _end
	// it  excludes the starter byte, checksum byte and end bytes
	// it starts at index i = _start and stops at index i = _end
	// DATA_PACKETS_OFFSET_CHECKSUM is the offset of the checksum value, generally at _end + 1
	// ERROR CHECKING
	// Error checks total : 0
	// PARAMETERS
	// Parameters total : 3
	// byte* _dataBuffer - Data in which checksum is to be calculated
	// uint8_t _start - offset start into _dataBuffer
	// uint8_t _end - offset end into _dataBuffer
	// RETURNS
	// Returns : bool - success/failure
	// PROGRAM DOC END DataPackets::VerifyChecksum()
	// PROGRAM FLOW START calling function: DataPackets::VerifyChecksum()

#ifdef _DEBUG_DATA_PACKETS_VERIFY_CHECKSUM
	PRINT_FUNCTION_ENTER_NL
		PRINT_VALUE_EQUAL_TAB(_start)
		PRINT_VALUE_EQUAL_TAB(_end)
#endif

		// development, usually doing checksum on packet buffer

		uint8_t calculatedChecksum = CalculateChecksum(_packet, _start, _end);
	bool verification{ true };
	if (calculatedChecksum != (uint8_t)_packet[_packetSize - 2]) {
#ifdef _DEBUG_DATA_PACKETS_VERIFY_CHECKSUM
		Serial.println(F("Checksum problem, returning false"));
		Serial.print(F("\tcalculatedChecksum "));
		Serial.println(calculatedChecksum);
		Serial.print(F("\t_dataBuffer "));
		Serial.println(_packet[endCodeChecksum]);
#endif
		verification = false;
	}
#ifdef _DEBUG_DATA_PACKETS_VERIFY_CHECKSUM
	Serial.println(F("exit normal"));
#endif
	return verification;
	// PROGRAM FLOW END calling function: DataPackets::VerifyChecksum()
} // VerifyChecksum

// End DataPackets.cpp
