#ifndef EVENT_STORAGE_H
#define EVENT_STORAGE_H

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

// This framework enables access to bool data packed as 1-bit bitfields inside 7 or 8 bit bytes
// to optimize data storage.
// The Events support class defines a list of events - it is application dependent.

extern char* myEventLabels[];

class EventStorage {
  public:
//-------------------------------- CONSTRUCTOR -------------------------------
      EventStorage(uint8_t _registerSize, uint8_t _eventCount, char* _eventLabels[]);
      ~EventStorage ( ) {delete eventsArray; };
//-------------------------------- GET/SET FUNCTIONS -------------------------------
    virtual bool Set(uint8_t _itemNumber, uint8_t _valueToSet);
    virtual bool Get(uint8_t _itemNumber, uint8_t& _valueToGet);
    virtual bool Reset();

// ------------------------------ AUXILIARY PUBLIC FUNCTIONS -------------------------
    virtual void PrintVariables();
    virtual void PrintRawData();
    virtual void PrintMasks();
    virtual bool VerifyMasks();
//-------------------------------- AUXILIARY PRIVATE FUNCTIONS ---------------------------------
    //private:
    uint8_t GetOffsetInMap(uint8_t _itemNumber);
    uint8_t GetSequenceIndex(uint8_t _itemNumber);
    uint8_t GetOffsetInSequence(uint8_t _itemNumber);
    uint8_t CreateMask( uint8_t _itemNumber, uint8_t& _mask);
//-------------------------------- PRIVATE VARIABLES ---------------------------------
  private:
    //Events events;
    const uint8_t registerSize{8};
    const uint8_t eventCount{0}; ; // contains nomber of events 
    const uint8_t* eventsArray{(uint8_t*) 0 }; // maps values of bitfields
    char** eventLabels;
    }; // EventStorage
    
#endif // EVENT_STORAGE_H
