/* ATest.cpp
   called by Aardvark to test stuff
*/

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"


// Scenario
// The scenario is my garden; I want to keep track of the the types of greenery it
// contains, be they flowers, grass varieties, trees, shrubs, whatever. Since it can be
// a big time consuming project, I'll start with flowers. 
// A top down approach is to think in terms of objects: flowers,
// shrubs, tress, etc. So, I created a Flower class. The next issue was how to store 
// these objects. Arrays are often the best solution since they are simple to set up 
// and use. The problem is their lack of flexibility: their size is fixed (sure, you
// can realloc but that is fraught with dangers) and moving items from one location in
// the array to another location requires some special attention. Furthermore, doing sorts 
// on arrays can be a hassle. Arrays are good when you know how many objects you need to store 
// and when you can easily provide an index into the array to access it. If you need 
// to shuffle things around or increase the size of the array, they may not be as good. 
// Linked list provide flexibility: sort, shuffle things around, not constrained ny the 
// number of itesm.
// LinkedList Framework to the rescue, I made it so that my Flower class would inherit
// from the LinkedListItem class. I also created a Garden class which inherits from
// the LinkedListManage class. 
//
// Notice the use of virtual print functions in the base linked list classes. These 
// enable prints to be customized in derived classes. 
// 
// In the scenario below, I created six flowers, added them to my garden, and printed 
// the garden's content. A lot more could be done once the garden has been populated;
// however, this remains as TODO work - see comments in the top of the LinkedList.cpp file.
//
// I stopped developing these classes to get the books published - work to do:
//    Put flower characteristics inside a struct to access them via an offset into
//    the struct. This would enable a generalized sort based on any of the characteristics
//    of a flower.


Garden::Garden(char* _name) 
	: name(_name), LinkedListManage(_name) {

}

char* definesName[]{ "family", "size", "color", "scent", "odorStrength", "plantType" };
char* familyName[]{ "Rose", "Sunflower", "Hydrangea", "Lavender", "Peony", "Daisy", "Orchid", "Tulip", "Lily", "Lilac", "Chrysanthemum", "Dahlia", "Daffodil", "Iris", "Periwinkle", "Azalea", "Carnation", "Violet", "end" };
char* sizeName[] { "small", "medium", "large", "end" };
char* nameColor[] { "blue", "green", "lavender", "orange", "pink", "purple", "red", "white", "yellow", "tan", "end" };
char* nameScent[] { "fragrant", "woody", "minty", "sweet", "chemical", "popcorn", "lemon", "fruity", "pungent", "decayed", "end" };
char* nameOdorStrength[] { "weak", "medium", "strong", "end" };
char* namePlantType[] { "stem", "bush", "thicket", "end" };

Flower::Flower(char* _name, EFamily _family, EPlantType _plantType, ESize _size, EColor _color, EScent _scent, EOdorStrength _odorStrength)
 : name(_name), 
	 family(_family), plantType(_plantType), size(_size), color(_color), scent(_scent), odorStrength(_odorStrength) {
}

void Flower::PrintItem() {

	Serial.print(F("Flower: "));
	Serial.print(name);
	Serial.print(F("\t"));
	Serial.print(definesName[0]);
	Serial.print(F(": "));
	Serial.println(familyName[(uint8_t)family]);

}

void ATest() {

	// Create some flowers
	// Define flower name, family, size, color, scent, odor power
	Flower elisabethRose = Flower("Elisabeth", EFamily::Rose, EPlantType::bush, ESize::large, EColor::yellow, EScent::fragrant, EOdorStrength::strong);
	Flower lillyRose = Flower("Lilly", EFamily::Rose, EPlantType::bush, ESize::large, EColor::red, EScent::fragrant, EOdorStrength::weak);
	Flower purpleLilac = Flower("PurpleExtravaganza", EFamily::Lilac, EPlantType::thicket, ESize::large, EColor::purple, EScent::sweet, EOdorStrength::weak);
	Flower whiteLilac = Flower("WhiteExtravaganza", EFamily::Lilac, EPlantType::thicket, ESize::large, EColor::white, EScent::sweet, EOdorStrength::weak);
	Flower blueIris = Flower("BlueSeeMe", EFamily::Iris, EPlantType::thicket, ESize::large, EColor::white, EScent::none, EOdorStrength::weak);
	Flower whiteIris = Flower("WhiteSeeMe", EFamily::Iris, EPlantType::thicket, ESize::large, EColor::white, EScent::none, EOdorStrength::weak);
	// Create Garden
	Garden myGarden = Garden("chezMoi"); // That's because I am French
	// Now add flowers to the garden
	myGarden.AddItem(&elisabethRose);
	myGarden.AddItem(&purpleLilac);
	myGarden.AddItem(&blueIris);
	myGarden.AddItem(&whiteLilac);
	myGarden.AddItem(&lillyRose);
	myGarden.AddItem(&whiteIris);
	myGarden.PrintList();
} // ATest()

// END FILS ATest.cpp
