#ifndef LINKED_LIST_H
#define LINKED_LIST_H

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

// Linked list classes

// class of individual items
class LinkedListItem {
	// Class of items which are to be linked one after the other and managed by a LinkedList object
	// previous and next are managed by the LinkedList
	// contains no data - if data is desired, create a class with data and have it inherit from the LinkedListItem class, an example class PeakItem is included
public:
	LinkedListItem() {}; // this constructor creates an item
	~LinkedListItem() {}; //item destructor
	LinkedListItem* GetNext() { return next; }
	LinkedListItem* GetPrevious() { return previous; }
	void SetNext(LinkedListItem* _next) { next = _next; }
	void SetPrevious(LinkedListItem* _previous) { previous = _previous; }
	virtual void PrintItem(); // Prints basic info - derived class prints its contents if defined
private:
	LinkedListItem* next;
	LinkedListItem* previous;
};

// class which manages the list
class LinkedListManage {
	// class which manages list of LinkedListItems.
	// Performs many operations on the list, namely : Add, Remove, Insert, Sort
	// Get Head/Tail - auxiliary functions to retrieve head/tail
public:
	LinkedListManage(char* _listStringName); // default constructor
	~LinkedListManage() {};
	uint16_t GetItemCount() { return count; }
	LinkedListItem* GetHead() { return head; }
	LinkedListItem* GetTail() { return tail; }
	bool FindItem(char* _name); // determins whether item is in the list
	void AddItem(LinkedListItem* _newItem); // adds an item at the end of the list
	void InsertItem(LinkedListItem* _existingItem, LinkedListItem* _newItem); // appends item after existing item
	bool SwapItems(LinkedListItem* _currentItem); // swaps the position of existingItem with its next neighbor.
	void DeleteItem(LinkedListItem* _currentItem);  // Context is three items : previous, existing to be deleted and following. _existingItem is deleted
	bool CheckListCoherence(char* _comment); // Undertake several checks to make sure list is OK
	bool ReverseList(); // Reverses the list. head becomes tail and vice versa
	void PrintList(); // Prints the content
	virtual void PrintItem(LinkedListItem* _existingItem) {}; // serves to debug - prints the content
private:
	char* name;
	LinkedListItem* head; // First item in the list
	LinkedListItem* tail; // Last item
	uint16_t count; // count of items in the list
};


#endif // LINKED_LIST_H
