// Begin ClassNames.cpp

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

#ifdef USE_CLASS_NAMES

#define _DEBUG_CLASS_NAME_LIST_ON
#ifdef _DEBUG_CLASS_NAME_LIST_ON
#define _DEBUG_CLASS_NAME_LIST_GET_CLASS_INDEX
#define _DEBUG_CLASS_NAME_LIST_GET_CLASS_NAME
#define _DEBUG_CLASS_NAME_LIST_GET_FUNCTION_INDEX
#define _DEBUG_CLASS_NAME_LIST_GET_FUNCTION_NAME
#endif // _DEBUG_CLASS_NAME_LIST_ON



char* ClassNames::GetClassName(EClass _classIndex) {
	// index is zero based
	// Index 0 is "Undefined", used for error checking
	// classes begin add index 1
#ifdef _DEBUG_CLASS_NAME_LIST_GET_CLASS_NAME
	PRINT_FUNCTION_ENTER_NL
		PRINT_VALUE_EQUAL_TAB((uint16_t)_classIndex)
#endif // _DEBUG_CLASS_NAME_LIST_GET_CLASS_NAME

		if (CheckClassIndex(_classIndex) == false) {
#ifdef _DEBUG_CLASS_NAME_LIST_GET_CLASS_NAME
			Serial.print(F("Error: CheckClassIndex() returned false"));
			PRINT_FUNCTION_EXIT
#endif // _DEBUG_CLASS_NAME_LIST_GET_CLASS_NAME
				return structCandFNames[0].classLabels;
		}

#ifdef _DEBUG_CLASS_NAME_LIST_GET_CLASS_NAME
	PRINT_FUNCTION_EXIT
#endif // _DEBUG_CLASS_NAME_LIST_GET_CLASS_NAME

		return structCandFNames[(uint16_t)_classIndex].classLabels;
} // GetName

uint16_t ClassNames::GetNumberOfClasses() {
	return (uint16_t)EClass::end;
} // GetNumberOfClasses ()

bool ClassNames::CheckClassIndex(EClass _classIndex) {
	uint16_t numberOfClasses = GetNumberOfClasses();
	if ((uint16_t)_classIndex >= numberOfClasses) {
		Serial.print(F("Error: class name index should be between 0 and "));
		Serial.println(numberOfClasses);
		return false;
	} // if
	return true;
}

uint8_t ClassNames::GetNumberOfFunctions(EClass _classIndex) {
	uint16_t sizeArray{ structCandFNames[(uint16_t)_classIndex].classLabels };
	uint8_t sizeItem{ structCandFNames[(uint16_t)_classIndex].functionLabels[0] };
	return sizeof(sizeArray) / sizeof(sizeItem);

} // GetNumberOfFunctions ()

bool ClassNames::CheckFunctionIndex(EClass _classIndex, uint8_t _functionIndex) {
	uint16_t numberOfClasses = GetNumberOfClasses();
	if (CheckClassIndex(_classIndex) == false) {
		Serial.print(F("Error: classIndex = "));
		Serial.println((uint16_t)_classIndex);
		Serial.print(F(", it should be between 0 and "));
		Serial.println(numberOfClasses);
		return false;
	} // if
	uint8_t numberOfFunctions = GetNumberOfFunctions(_classIndex);
	if (CheckFunctionIndex(_classIndex, _functionIndex) == false) {
		Serial.print(F("Error: functionIndex = "));
		Serial.println(_functionIndex);
		Serial.print(F(", it should be between 0 and "));
		Serial.println(numberOfFunctions);
		return false;
	} // if
	return true;
} // CheckFunctionIndex ()

char* ClassNames::GetFunctionName(EClass _classIndex, uint8_t _functionIndex) {
	// Check
	if (CheckClassIndex(_classIndex) == false) {
		Serial.println(F("Error: ClassAndFunctionNames - class index out of range"));
	}
	if (structCandFNames[(uint16_t)_classIndex].classLabels == 0) {
		Serial.println(F("Error: ClassAndFunctionNames - Function names array is null"));
		return 0;
	}
	return structCandFNames[(uint16_t)_classIndex].functionLabels[_functionIndex];
}; // GetFunctionName

void ClassNames::PrintClassList() {
	uint8_t numberOfClasses{ 0 };
	for (int i = 0; i < GetNumberOfClasses(); i++) {
		Serial.print(F("  i = "));
		Serial.print(i);
		Serial.print(F(" : "));
		Serial.print(structCandFNames[i].classLabels[i]);
		Serial.println();
	} // for
	return;
} // PrintClassList ()

void ClassNames::PrintFunctionList(EClass _classIndex) {
	Serial.print(F("Printing functions for class "));
	Serial.println(GetClassName(_classIndex));
	for (int i = 0; i < GetNumberOfFunctions(_classIndex); i++) {
		Serial.print(F("  i = "));
		Serial.print(i);
		Serial.print(F(" : "));
		Serial.print(structCandFNames[(uint16_t)_classIndex].functionLabels[i]);
		Serial.println();
	} // for
} // PrintFunctionList ()

#endif // USE_CLASS_NAMES


// End ClassNames.cpp
