// Begin ErrorRpt.cpp

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

#ifdef USE_ERROR_RPT

#define _DEBUG_EVENT_MGT_ON
#ifdef _DEBUG_EVENT_MGT_ON
// ----------------------------------------  ERROR RPT ----------------------------------------
//#define _DEBUG_ERROR_RPT_EVENT_LIST_APPEND_ITEM
#define _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
#define _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_LIST

// ----------------------------------------  MEM ALLOC ----------------------------------------
//#define _DEBUG_MEM_ALLOC_EVENT_LIST_APPEND_ITEM
#define _DEBUG_MEM_ALLOC_EVENT_LIST_PRINT_ITEM
#define _DEBUG_MEM_ALLOC_EVENT_LIST_PRINT_LIST

// ----------------------------------------  MEMORY CONSTRUCTOR/DESTRUCTOR ----------------------------------------
//#define _DEBUG_CALCULATE_OBJECT_FU_SIZE
#define _DEBUG_HANDLE_MEMORY_CONSTRUCTOR
//#define _DEBUG_HANDLE_MEMORY_DESTRUCTOR
#define _DEBUG_HEAP_SPACE_DIAGNOSTICS
#endif // _DEBUG_MEM_MGT_ON


#ifdef _DEBUG_EVENT_MGT_ON


char* eErrorTypeLabels[] {
  "outOfBounds",
  "badAlloc",
  "Allocated",
  "Released"
};

#endif // _DEBUG_EVENT_MGT_ON

void ErrorRptList::AppendItem (ErrorRptEvent* _errorRptEvent) {
  #ifdef _DEBUG_ERROR_RPT_EVENT_LIST_APPEND_ITEM
  PRINT_FUNCTION_ENTER_NL
  Serial.println (F ("Appending item to ERROR list"));
  #endif _DEBUG_ERROR_RPT_EVENT_LIST_APPEND_ITEM
  
  errorRptList->AppendItem (_errorRptEvent);
  
  #ifdef _DEBUG_ERROR_RPT_EVENT_LIST_APPEND_ITEM
  PRINT_FUNCTION_EXIT
  #endif _DEBUG_ERROR_RPT_EVENT_LIST_APPEND_ITEM
  return;
}; // ErrorRptEventMgt


void ErrorRptList::PrintList (){
  #ifdef _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_LIST
  PRINT_FUNCTION_ENTER_NL
  #endif // _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_LIST

  #ifdef PRINT_WHEN_ALL_ELSE_OFF
  Serial.println (F ("Errors detected:"));
  PrintItems ();
  #endif // PRINT_WHEN_ALL_ELSE_OFF

  #ifdef _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_LIST
  PRINT_FUNCTION_EXIT
  #endif // _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_LIST
  return;
} // PrintList ()

void ErrorRptEvent::PrintItem (bool _insertNewLine) {
  #ifdef _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
  PRINT_FUNCTION_ENTER_NL
  #endif // _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
  //uint8_t functionIndex = (uint8_t) EErrorRptEvent::PrintItem;
  uint8_t functionIndex = (uint8_t) 1;
  #ifdef PRINT_WHEN_ALL_ELSE_OFF
  #ifdef USE_CLASS_NAMES
  Serial.print (F ("  "));
  Serial.print (classAndFunctionNames->GetClassName (classIndex));
  Serial.print (F ("::"));
  Serial.print (classAndFunctionNames->GetFunctionName (classIndex, functionIndex));
  Serial.print (F (":"));
  #else
  Serial.print (F ("  classIndex/functionIndex "));
  Serial.print (classIndex);
  Serial.print (F ("/"));
  Serial.print (functionIndex);
  Serial.print (F (" lineNo "));
  #endif
  Serial.print (GetLineNo ());
  Serial.print (F (", status "));
  Serial.print (eErrorTypeLabels[ (uint8_t)status]);
  if ( _insertNewLine == true ) {
    Serial.println ();
  }
  #endif // PRINT_WHEN_ALL_ELSE_OFF
  
  #ifdef _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
  PRINT_FUNCTION_EXIT
  #endif // _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
  return;
} // PrintItem ;

 
// ----------------------------------------  MEMORY CONSTR!UCTOR/DESTRUCTOR ----------------------------------------

#endif // USE_ERROR_RPT

// End ErrorRpt.cpp
