// Begin Personnel.cpp

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol (France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

//#define _DEBUG_PERSONNEL_ON
#ifdef _DEBUG_PERSONNEL_ON
#define _DEBUG_PERSONNEL_EMPLOYEE_CONSTRUCTOR
#define _DEBUG_PERSONNEL_COMPANY_CONSTRUCTOR

#endif // _DEBUG_PERSONNEL_ON
char* deptNames[]{
	"Unassigned", "Electrical", "Research", "Data processing", "Administration", "Production"
};

Employee::Employee(char* _name, EDept _dpt, uint16_t _itemID)
	: dpt(_dpt), LinkedListItem(_name, _itemID, CLASS_INDEX_USE(Employee)) {
#ifdef _DEBUG_PERSONNEL_EMPLOYEE_CONSTRUCTOR
	PRINT_FUNCTION_ENTER_NL;
	PRINT_VALUE_EQUAL_NL((char*)_name);
#endif // _DEBUG_PERSONNEL_CONSTRUCTOR

#ifdef _DEBUG_PERSONNEL_EMPLOYEE_CONSTRUCTOR
	PRINT_FUNCTION_EXIT_NL;
#endif // _DEBUG_PERSONNEL_EMPLOYEE_CONSTRUCTOR

} // Employee::Employee

Company::Company(char* _name)
	: LinkedListManage(_name, 0, EClass::Company) {
#ifdef _DEBUG_PERSONNEL_COMPANY_CONSTRUCTOR
	PRINT_FUNCTION_ENTER_NL;
	PRINT_VALUE_EQUAL_NL((char*)_name);
#endif // _DEBUG_PERSONNEL_COMPANY_CONSTRUCTOR
#ifdef _DEBUG_PERSONNEL_COMPANY_CONSTRUCTOR
	PRINT_FUNCTION_EXIT_NL;
#endif // _DEBUG_PERSONNEL_COMPANY_CONSTRUCTOR
} // Company::Company

void Employee::PrintItem() {
	Serial.print(GetName());
	Serial.print(F("  - ID: "));
	Serial.print(GetItemID());
	Serial.print(F(" address "));
	Serial.print((long)this);
	Serial.print(F(" Dept. "));
	Serial.print(deptNames[(uint8_t)dpt]);
#ifdef USE_MEM_ALLOC_TRACKING
	Serial.print(F(" Unique mem tracking no. "));
	Serial.print(GetItemID());
#endif // USE_MEM_ALLOC_TRACKING
	Serial.println();
} // Employee::PrintItem

void Company::AddEmployee(Employee* _employee) {
	AddItem(_employee);
} // Company::AddItem



// End Personnel.cpp
