// Begin LinkedList.cpp

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol (France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michèle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.


// Linked list classes
// The LinkedList Framework comprises two classes: a class of items which are to be 
// linked one after the other and a list class which manages insertions, printing, sorts, etc.
// The LinkedListItem class has next and previous atributes to enable navigation up and down the list.
// The LinkedListManage class has head and tail attributes to enable traversing the list forwards and backwards.
// Both classes posses virtual print functions so that derived methods could customize prints. 
// 
// I stopped developing these classes to get the books published - work to do:
// 1 -	Insert, swap, check for item have not been verified - they probably need to be reprogrammed.

#include "Globals.h"

// ITEMS
//#define _DEBUG_LINKED_LIST_ITEM_PRINT_ITEM

// LIST
//#define _DEBUG_LINKED_LIST_MANAGE_PRINT_CONTENTS
//#define _DEBUG_LINKED_LIST_ITEM_PRINT_ITEM
//#define _DEBUG_LINKED_LIST_MANAGE_ADD_ITEM

#define ASCENDING 1 // these serve to determine direction of sort of linked list.
#define DESCENDING 0

LinkedListItemInfo::LinkedListItemInfo(char* _name, uint16_t _itemID, CLASS_INDEX_PARAMETER)
	: name(_name), itemID(_itemID) {
	name = new char[strlen(_name) + 1];
	strcpy(name, _name);
};

LinkedListItem::LinkedListItem(char* _name, uint16_t _itemID, CLASS_INDEX_PARAMETER)
	: LinkedListItemInfo(_name, _itemID, CLASS_INDEX_PARAMETER_TRANSFER) {
}

void LinkedListItem::PrintItem() {
	Serial.print(GetName());
#ifdef _DEBUG_LINKED_LIST_ITEM_PRINT_ITEM
	// serves to debug
	Serial.print(F("LinkedListItem - PrintContents()"));
	Serial.print(F(" / next pointer     : "));
	Serial.print((uint16_t)next);
	Serial.print(F(" / previous pointer : "));
	Serial.println((uint16_t)previous);
#endif //_DEBUG_LINKED_LIST_ITEM_PRINT_ITEM
} // LinkedListItem::PrintItem


// LIST
LinkedListManage::LinkedListManage(char* _listStringName, uint16_t _itemID, CLASS_INDEX_PARAMETER)
	: LinkedListItemInfo(_listStringName, _itemID, CLASS_INDEX_PARAMETER_TRANSFER) {
	// Create empty list with head and previous
	head = new LinkedListItem("", 0, EClass::none);
	tail = new LinkedListItem("", 0, EClass::none);
	count = 0;
	head->SetPrevious(0);
	head->SetNext(tail);
	tail->SetPrevious(head);
	tail->SetNext(0);
} // LinkedListManage::LinkedListManage


void* LinkedListManage::FindItem(void* _itemPtr) {
	// Traverse link list - implement in derived classes
	Serial.println(F("LinkedListManage::FindItem Error - derived class verssoin should be called."));
	return 0;
} //CheckForItem()

void LinkedListManage::AddItem(LinkedListItem* _newItem) {
	// adds an item to the end of the list
	// Inserts it between the last item and tail.
#ifdef _DEBUG_LINKED_LIST_MANAGE_ADD_ITEM
	PRINT_FUNCTION_ENTER_NL
		PRINT_VALUE_EQUAL_DESC_NL("LinkedList - Adding item : ", (long)_newItem);
#endif

	// handle tail and previous item
	LinkedListItem* previousItem = tail->GetPrevious();
	tail->SetPrevious(_newItem);
	previousItem->SetNext(_newItem);

	// handle item being added
	_newItem->SetPrevious(previousItem);
	_newItem->SetNext(tail);

	// Update count
	count++;
	itemID++;
#ifdef _DEBUG_LINKED_LIST_MANAGE_ADD_ITEM
	PRINT_VALUE_EQUAL_TAB_NL((long)previousItem);
	PRINT_VALUE_EQUAL_TAB_NL((long)_newItem);
	PRINT_VALUE_EQUAL_TAB_NL((long)tail);
	PRINT_VALUE_EQUAL_TAB_NL((long)previousItem->GetNext());
	PRINT_VALUE_EQUAL_TAB_NL((long)_newItem->GetPrevious());
	PRINT_VALUE_EQUAL_TAB_NL((long)_newItem->GetNext());
#endif

#ifdef _DEBUG_LINKED_LIST_MANAGE_ADD_ITEM
	PRINT_FUNCTION_EXIT_NL
#endif
} // AddItem()

void LinkedListManage::InsertItem(LinkedListItem* _existingItem, LinkedListItem* _newItem) {
	// appends item after existing item
	// if you want to insert an item as the new head, first do an appendItem after the head and then do a swapItem from the head.
#ifdef DEBUG
	Serial.println(F("LinkedList - AppendItem"));
#endif
	// check whether appending at tail - inserts a new item after existing item
	// Since appending on empty list, will insert as if it were an add item.
	LinkedListItem* existingItem = _existingItem;
	LinkedListItem* newItem = _newItem;
	if (head == NULL) {
#ifdef DEBUG
		Serial.println(F("LinkedList - AppendItem - head = null, SHOULD NOT HAPPEN. Not inserting : "));
		Serial.print((uint16_t)_newItem);
		delay(10000);
#endif
		return;
	}
	if (_existingItem == NULL) {
#ifdef DEBUG
		existingItem->PrintContents("LinkedList - AppendItem - coherence problem. head = null yet existingItem not equal null.");
#endif
		delay(10000);
		return;
	}
	else { // OK from here on - inserting somewhere inside the list or at the end.
#ifdef DEBUG
		Serial.println(F("LinkedList - AppendItem - finding tail to check whether inserting at tail."));
#endif
		LinkedListItem* temp = NULL;
#ifdef DEBUG
		Serial.print(F("LinkedList - AppendItem - found tail - address : "));
		Serial.println((uint16_t)temp);
#endif
		existingItem->SetNext(newItem);
		newItem->SetPrevious(existingItem);
		// check whether appending to tail
		if (existingItem == tail) { // appending at tail
			newItem->SetNext(NULL);
			tail = _newItem;
		}
		else { // inserting somewhere inside the list
			temp = existingItem->GetNext();
			newItem->SetNext(temp);
			temp->SetPrevious(newItem);
		}
		count++;
	}
} // LinkedListManage::InsertItem

bool LinkedListManage::SwapItems(LinkedListItem* _currentItem) {
	// NEEDS VERIFYING
	// Swaps _currentItem with item after it 
	//			Before swap After swp
	// next next	NN			NN
	// next			N			 C
	// current		C			 N
	// previous		P			 P
	//

	LinkedListItem* current = _currentItem;
	LinkedListItem* previous = NULL;
	LinkedListItem* nextNext = NULL;
	LinkedListItem* next = NULL;
	LinkedListItem* tempItem = NULL;

	if (_currentItem == head || _currentItem == tail || _currentItem->GetNext() == tail) { // do nothing except print message in debug mode
		return false; // could return true but there must be some programming flaw somewhere
	}

	// Get pointers
	previous = current->GetPrevious();
	next = current->GetNext();
	nextNext = next->GetNext();

	// Set next/previous
	previous->SetNext(next);
	next->SetPrevious(previous);
	next->SetNext(current);
	current->SetPrevious(next);
	current->SetNext(nextNext);
	nextNext->SetPrevious(current);
} // LinkedListManage::SwapItems

void LinkedListManage::DeleteItem(LinkedListItem* _currentItem) {
	// NEEDS VERIFYING
	// Deletes item  
	//			Before swap After swp
	// next			N			 N
	// current		C		   deleted
	// previous		P			 P
	//

	LinkedListItem* current = _currentItem;
	LinkedListItem* previous = NULL;
	LinkedListItem* next = NULL;
	LinkedListItem* tempItem = NULL;

	previous->SetNext(next);
	next->SetPrevious(previous);
	count--;
	delete(_currentItem);
} // LinkedListManage::DeleteItem

bool LinkedListManage::CheckListCoherence(char* _comment) {
	// NEEDS VERIFYING
	// Undertake several checks to make sure list is OK
	uint16_t delayTime = 1000;
	bool errorFound = false;
#ifdef DEBUG
	Serial.print(F("CheckListCoherence() - entering from : "));
	Serial.println(_comment);
#endif
	// Check for number of items
	if (count <= 0) {
#ifdef DEBUG
		Serial.print(F("Requesting CheckListCoherence() on a list whose number of items = "));
		Serial.println(itemCount);
#endif
		//      SoundBuzzAndWait();
		errorFound = true;;
	}
	// Check for existence of head and tail
	if (head == NULL) {
#ifdef DEBUG
		Serial.print(F("CheckListCoherence() : head is NULL, should not be as item count > 0. "));
		Serial.println(itemCount);
		delay(delayTime);
#endif
		errorFound = true;;
	}
	if (tail == NULL) {
#ifdef DEBUG
		Serial.print(F("CheckListCoherence() : tail is NULL, should not be as there is a head. "));
		Serial.println(itemCount);
		delay(delayTime);
#endif
		errorFound = true;;
	}
	// Check for  GetPrevious & GetNext on head an tail
	if (head->GetPrevious() != NULL) {
#ifdef DEBUG
		Serial.println(F("CheckListCoherence() : head->GetPrevious() should be NULL. No so!."));
		delay(delayTime);
#endif
		errorFound = true;;
	}
	if (tail->GetNext() != NULL) {
#ifdef DEBUG
		Serial.print(F("CheckListCoherence() : tail->GetNext() should be NULL. No so!."));
		Serial.println(itemCount);
		delay(delayTime);
#endif
		errorFound = true;;
	}
	// Check itemCount when head and tail are the same
	if (head == tail && count != 1) {
#ifdef DEBUG
		Serial.print(F("CheckListCoherence() : head and tail identical yet itemCount not 1. number of items = "));
		Serial.println(itemCount);
		delay(delayTime);
#endif
		errorFound = true;;
	}
	// Check item count
	LinkedListItem* temp = head;
	int itemCount = 0;
	while (true) {
		itemCount++;
		temp = temp->GetNext();
		if (temp == NULL) break;
	}
	if (itemCount != count) {
#ifdef DEBUG
		Serial.print(F("CheckListCoherence() : problem with itemCount. counted "));
		Serial.print(count);
		Serial.print(F("against temCount = "));
		Serial.println(itemCount);
#endif
	}
	if (errorFound == true) {
		delay(30000);
	}
	return errorFound;
} // LinkedListManage::CheckListCoherence

bool LinkedListManage::ReverseList() {
	// NEEDS IMPLEMENTIG

	return false;
} // LinkedListManage::ReverseList()

void LinkedListManage::PrintList() {
#ifdef _DEBUG_LINKED_LIST_MANAGE_PRINT_CONTENTS
	PRINT_FUNCTION_ENTER_NL;
#endif

	if (count == 0) {
		Serial.println(F("LinkedList - PrintContents() - Sorry, list is empty."));
	}
	else {
		// Get first item in list
		Serial.print(F("List "));
		Serial.print(GetName());
		Serial.print(F(" has "));
		Serial.print(count);
		Serial.println(F(" items."));

		LinkedListItem* tempListItem = head->GetNext();

		// Traverse the list from head to tail using count as a safety
		// in case null gets thrashed
		uint16_t tempCount = 0;
		while (tempListItem != tail && tempCount <= count) {
			tempCount++;
			Serial.print(F("\t"));
			Serial.print(tempCount);
			Serial.print(F("\t"));
			tempListItem->PrintItem();
			// check whether encountered tail item, if not cycle once more
			if (tempListItem->GetNext() == tail) {
				break;
			}
			else {
				tempListItem = tempListItem->GetNext();
			}
		}
		Serial.println();
	}
#ifdef _DEBUG_LINKED_LIST_MANAGE_PRINT_CONTENTS
	PRINT_FUNCTION_EXIT_NL;
#endif
} // LinkedListManage::PrintList

// End LinkedList.cpp


