// Begin MemMgt.cpp

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol (France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

#ifdef USE_MEM_ALLOC_TRACKING

//#define _DEBUG_MEM_MGT_ON
#ifdef _DEBUG_MEM_MGT_ON
#define _DEBUG_MEM_MGT_MEM_ALLOC_CONSTRUCTOR
//#define _DEBUG_MEM_MGT_DESTRUCTOR
//#define _DEBUG_MEM_MGT_GET_FREE_SRAM
//#define _DEBUG_MEM_MGT_FREE_SRAM
//#define _DEBUG_MEM_MGT_CHECK_MEMORY_BEFORE_NEW
//#define _DEBUG_MEM_MGT_HEAP_SPACE
//#define _DEBUG_MEM_MGT_REPORTING_MEM_CREATE_MEMM_ALLOC_ITEM
//#define _DEBUG_MEM_MGT_REPORTING_MEM_ALLOC_DESTROY
//#define _DEBUG_MEM_MGT_PRINT
//#define _DEBUG_SIMPLE_PRINT_HEAP_SPACE


// ----------------------------------------  FOLLOW UP ----------------------------------------
//#define _DEBUG_OBJECT_CREATION_FOLLOW_UP_LIST_APPEND_ITEM
#define _DEBUG_OBJECT_CREATION_FOLLOW_UP_LIST_PRINT

// ----------------------------------------  MEMORY CONSTRUCTOR/DESTRUCTOR ----------------------------------------
#define _DEBUG_CALCULATE_OBJECT_FU_SIZE
#define _DEBUG_HANDLE_MEMORY_CONSTRUCTOR
#define _DEBUG_HANDLE_MEMORY_DESTRUCTOR

#endif // _DEBUG_MEM_MGT_ON

char* eAllocStatusLabels[]{
	"allocated",
	"deleted"
};

char* eAllocTypeLabels[]{
	"undefined",
	"global",
	"uint16_t",
	"char",
	"Employee",
	"Company"
};

MemAllocTrackingItem::MemAllocTrackingItem(
	uint16_t _itemID, uint16_t* _itemAddress,
	EAllocStatus _allocStatus, EAllocType _allocType, EAllocType _allocContext 
	) : itemAddress(_itemAddress), LinkedListItem("", _itemID, CLASS_INDEX_USE(none)),
		allocStatus(_allocStatus), allocType(_allocType), allocContext(_allocContext) {
#ifdef _DEBUG_MEM_MGT_MEM_ALLOC_CONSTRUCTOR
	PRINT_FUNCTION_ENTER_NL
		PRINT_VALUE_EQUAL_TAB_NL((long)_itemAddress)
		PRINT_VALUE_EQUAL_TAB_NL(_itemID)
#endif // _DEBUG_MEM_MGT_MEM_ALLOC_CONSTRUCTOR

#ifdef _DEBUG_MEM_MGT_MEM_ALLOC_CONSTRUCTOR
		PRINT_FUNCTION_EXIT_NL
#endif // _DEBUG_MEM_MGT_MEM_ALLOC_CONSTRUCTOR
}

void MemAllocTrackingItem::PrintItem() {
#ifdef _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
	PRINT_FUNCTION_ENTER_NL;
#endif // _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
	char* itemName = ((LinkedListItem*)itemAddress)->GetName();
	Serial.print(F("\t"));	Serial.print(itemName);
	Serial.print(F("\t"));	Serial.print(GetItemID());
	Serial.print(F("\t"));	Serial.print((long)itemAddress);
	Serial.print(F("\t"));	Serial.print(eAllocStatusLabels[(uint8_t)allocStatus]);
	Serial.print(F("\t"));	Serial.print(eAllocTypeLabels[(uint8_t)allocType]);
	Serial.print(F("\t"));	Serial.print(eAllocTypeLabels[(uint8_t)allocContext]);
	Serial.println();
#ifdef _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
	PRINT_FUNCTION_ENTER_NL
#endif // _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
} // MemAllocTrackingItem::PrintItem

void MemAllocTrackingList::AddMemAlloc(MemAllocTrackingItem* _trackingAlloc) {
	AddItem(_trackingAlloc);
} // MemAllocTrackingList::AddItem

void* MemAllocTrackingList::FindItem(void* _itemPtr) {
	// Traverse list to find items
	Serial.print(F("MemAllocTrackingList::FindItem - looking for "));
	Serial.println((long)_itemPtr);
	int totalCount = memAllocTrackingList->GetCount();
	int runningCount{0};
	for (MemAllocTrackingItem* trackingListItem = memAllocTrackingList->GetHead();
		trackingListItem != memAllocTrackingList->GetTail();
		trackingListItem = trackingListItem->GetNext()
		) {
		if (trackingListItem->GetItemAddress() == _itemPtr) {
			// Got item
			return trackingListItem;
		}
		// Safety measure
		if (runningCount > totalCount) {
			Serial.println(F("MemAllocTrackingList::FindItem - Error - exceeded total count trying to find item."));
		}
		runningCount++;
	}
	return 0;
}

void MemAllocTrackingList::DeleteMemAlloc(MemAllocTrackingItem* _trackingAlloc) {
	_trackingAlloc->SetStatus(EAllocStatus::deleted);
}

void MemAllocTrackingList::PrintList() {
#ifdef _DEBUG_MEM_ALLOC_EVENT_LIST_PRINT_LIST
	PRINT_FUNCTION_ENTER_NL;
#endif // _DEBUG_MEM_ALLOC_EVENT_LIST_PRINT_LIST

	Serial.println(F("memory stats : "));
	Serial.print(F("  Total number of objects created = "));
	Serial.println(memAllocTrackingList->GetItemID());
	Serial.print(F("  Number of objects which have not released memory = "));
	Serial.println(memAllocTrackingList->GetCount());
	LinkedListManage::PrintList();

#ifdef _DEBUG_MEM_ALLOC_EVENT_LIST_PRINT_LIST
	PRINT_FUNCTION_EXIT
#endif // _DEBUG_MEM_ALLOC_EVENT_LIST_PRINT_LIST
		return;
} // MemAllocTrackingList::PrintList





#endif // USE_MEM_ALLOC_TRACKING

// End MemMgt.cpp
