// Begin OperatorOverloading.cpp

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol (France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

//--------------------------------- GLOBAL_OPERATOR_NEW ------------------------

//#define _DEBUG_OPERATOR_NEW_LINKED_LIST


#ifdef USE_MEM_ALLOC_TRACKING
void* operator new (size_t _allocSize, uint16_t _itemID, CLASS_INDEX_PARAMETER, EAllocStatus _status, EAllocType _type, EAllocType _context) {
#ifdef _DEBUG_OPERATOR_NEW_LINKED_LIST
	Serial.print(F("Entered global operator with allocSize = ")); Serial.println(_allocSize);
#endif // _DEBUG_OPERATOR_NEW_LINKED_LIST
	void* allocPtr = malloc(_allocSize);
	if (allocPtr == 0) {
#ifdef _DEBUG_OPERATOR_NEW_LINKED_LIST
		Serial.print(F("Error: Operator new alloc - class index = "));
#endif // _DEBUG_OPERATOR_NEW_LINKED_LIST
	}
	else {
#ifdef _DEBUG_OPERATOR_NEW_LINKED_LIST
		Serial.print(F("Success: Operator new alloc - class index = "));
#endif // _DEBUG_OPERATOR_NEW_LINKED_LIST
		MemAllocTrackingItem* memAllocPtr = new 
			MemAllocTrackingItem(memAllocTrackingList->GetItemID(), allocPtr, _status, _type, _context);
			//MemAllocTrackingItem(memAllocTrackingList->GetItemID(), (uint16_t*)bestHoney, EAllocStatus::allocated, EAllocType::Company, EAllocType::e_global);
		memAllocTrackingList->AddMemAlloc(memAllocPtr);
	}
#ifdef _DEBUG_OPERATOR_NEW_LINKED_LIST
	Serial.print(_classIndex);
	Serial.print(F(" - _allocSize = "));
	Serial.println(_allocSize);
#endif // _DEBUG_OPERATOR_NEW_LINKED_LIST
	return allocPtr;
} // operator new

void operator delete (void* _itemPtr) {
	Serial.println(F("Using overloaded delete operator"));
	MemAllocTrackingItem* memAllocItem{ 0 };
	memAllocItem = memAllocTrackingList->FindItem(_itemPtr);
	if (memAllocItem == 0) {
		Serial.print(F("overloaded delete - Error - could not find item address = "));
		Serial.println((long)_itemPtr);
		return;
	}
	// Found item - register it as being deleted then delete
	memAllocTrackingList->DeleteMemAlloc(memAllocItem);
} // operator delete (void* _itemID)
#endif // USE_MEM_ALLOC_TRACKING

// End OperatorOverloading.cpp
