/* ATest.cpp
  called by Aardvark to test stuff
*/

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

//Scenario:
//I have a small bee yard which I wish to keep track of. I consequently created bee yard and
// bee hive classes to manage them. I use the template SafeArray class to manage the 
// bee yard's array of beehives - it overloads the array index [] operator to safeguard
// against using out-of-bounds indices. I also overloaded the += operator to add hives 
// to the bee yard.
// 
// The bee yard can manage 4 bee hives. The scenario tries to add a fifth one. The SafeArray class does
// its job by preventing a fifth hive to be added to the yard. A longjmp gets triggered to inform
// on what is happening and to continue execution. It could have stopped the program by directing
// execution down to error processing. An error event is created; errors are printed before 
// the program exits.
// 
// Mechanisms used:
// BeeYard and BeeHive classes (linked List derived classes) manage the hives.
// Overloaded += operator to add hives to the bee yard.
// SafeArray class - overloads index operator '[]' to prevent out-of-bounds conditions.
// setjmp/longjmp redirect execution when an error occurs. 
// Error reporting class  (linked List derived classes) creates an event which is added 
// to the list of events to be printed when the program finishes.
// The ClassFunctionIDs files contain the strurure which enables printing 
// class/function/line no. of errors detected. Note that the ClassNames class is not used.
//

jmp_buf envAbort; // required for long jumps
ErrorRptList errorRptList = ErrorRptList("Michele's bee yard error reporting", 0, EClass::none);

void ATest() {

	PRINT_FUNCTION_ENTER_NL;

	BeeYard beeYard = BeeYard("Michele's bees");
	BeeHive* rosebud = new BeeHive("Rosebud"); BeeHive* beeHive = rosebud;
	BeeHive* mary = new BeeHive("Mary"); beeHive = mary;
	BeeHive* olivia = new BeeHive("Olivia"); beeHive = olivia;
	BeeHive* morgane = new BeeHive("Morgane"); beeHive = morgane;
	BeeHive* cynthia = new BeeHive("Cynthia"); beeHive = cynthia;
	uint8_t errorSetJmp = setjmp(envAbort);
	/// If error occurs when adding hive to yard, program jumps back to here
	if (errorSetJmp != 0) {
		//PRINT_VALUE_EQUAL_NL(errorSetJmp);
		//PRINT_VALUE_EQUAL_NL((uint8_t)EErrorType::outOfBounds);
		if (errorSetJmp == (uint8_t)EErrorType::outOfBounds) {
			Serial.print(F("Sorry, bee hive "));
			Serial.print(beeHive->GetName());
			Serial.println(F(" not added to bee yard, continuing."));
			Serial.print(F("Bee yard '"));
			Serial.print(beeYard.GetName());
			Serial.print(F("' limited to "));
			Serial.print(beeYard.GetMaxNumberOfHives());
			Serial.println(F(" hives."));
			//goto envAbort;
		}
	} else {
	beeYard += rosebud;
	beeYard += mary;
	beeYard += olivia;
	beeYard += morgane;
	beeYard += cynthia;
	} // if (errorSetJmp != 0) 

	beeYard.PrintList();
	errorRptList.PrintList();
	return;
envAbort:
	Serial.println(F("Encountered problem"));
	PRINT_FUNCTION_EXIT_NL;
	return;
} // ATest ()


// End ATest.cpp
