// Begin ErrorRpt.cpp

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

#define _DEBUG_ERROR_RPT_ON
#ifdef _DEBUG_ERROR_RPT_ON
// ----------------------------------------  ERROR RPT ----------------------------------------
//#define _DEBUG_ERROR_RPT_EVENT_LIST_APPEND_ITEM
//#define _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
#define _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_LIST

#endif _DEBUG_ERROR_RPT_ON

char* eErrorTypeLabels[] {
  "none",
  "outOfBounds",
  "badAlloc",
  "Allocated",
  "Released"
};

char* eEventTypeLabels[]{
  "none",
  "Error",
  "Event"
};

void ErrorRptList::AppendItem (ErrorRptItem* _errorRptEvent) {
  #ifdef _DEBUG_ERROR_RPT_EVENT_LIST_APPEND_ITEM
  PRINT_FUNCTION_ENTER_NL
  Serial.println (F ("Appending item to ERROR list"));
  #endif _DEBUG_ERROR_RPT_EVENT_LIST_APPEND_ITEM
  
  errorRptList.AppendItem (_errorRptEvent);
  
  #ifdef _DEBUG_ERROR_RPT_EVENT_LIST_APPEND_ITEM
  PRINT_FUNCTION_EXIT
  #endif _DEBUG_ERROR_RPT_EVENT_LIST_APPEND_ITEM
  return;
}; // ErrorRptEventMgt

void ErrorRptItem::PrintItem() {
#ifdef _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
    PRINT_FUNCTION_ENTER_NL;
#endif // _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
    switch (eventType)
    {
    case EEventType::error:
         Serial.print(F("Error: class "));
       break;
    default:
         Serial.print(F("Event: class "));
       break;
    } // switch
    Serial.print(structCandFNames[(uint16_t)classIndex].classLabel);
    Serial.print(F(" - function "));
    Serial.print(structCandFNames[(uint16_t)classIndex].functionLabels[functionIndex]);
    Serial.print(F(" - lineNo "));
    Serial.print(lineNo);
    Serial.println();

#ifdef _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
    PRINT_FUNCTION_EXIT_NL;
#endif // _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_ITEM
    return;
} // PrintItem ;

void ErrorRptList::PrintList (){
  #ifdef _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_LIST
    PRINT_FUNCTION_ENTER_NL;
  #endif // _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_LIST

    Serial.print(F("Error report list - Errors detected count :"));
    uint16_t count = GetCount();
    Serial.println(count);
    ErrorRptItem* errorItem = GetHead()->GetNext();
    if (count > 0) {
        for (; errorItem != GetTail(); errorItem = errorItem->GetNext()) {
            errorItem->PrintItem();
        }
    }

  #ifdef _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_LIST
  PRINT_FUNCTION_EXIT_NL
  #endif // _DEBUG_ERROR_RPT_EVENT_LIST_PRINT_LIST
  return;
} // PrintList ()

// ----------------------------------------  MEMORY CONSTR!UCTOR/DESTRUCTOR ----------------------------------------

// End ErrorRpt.cpp
