#ifndef ERROR_RPT_H
#define ERROR_RPT_H

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"
#ifdef USE_ERROR_RPT

//------------------------------------------------ ERROR EVENTS CREATION FOLLOW UP ------------------------------------------------------------

// As objects get created and deleted, details on object creation/deletion
// are stored in global array ErrorRptList::linkedList. This array's size is initalized to a fix value
// Each array item contains info plus the class name's address. The class name's is contained in a link list
// of class names progressively built up as items are created.


//--------------------------------  ERROR EVENT F/U ----------------------------------------------------
enum class EEventType : uint8_t {
	start = 0,
	none = 0,
	error,
	event,
	end
};
enum class EErrorType : uint8_t {
	start = 0,
	none = 0, // retires contact from data base
	outOfBounds, // retires contact from data base
	badAlloc,
	allocated,
	released,
	end
}; // enum EErrorType

extern char* eEventTypeLabels[];
extern char* eErrorTypeLabels[];

class ErrorRptItem : public LinkedListItem {
public:
	ErrorRptItem(EEventType _eventType, EErrorType _errorType, EClass _classIndex, uint8_t _functionIndex, uint16_t _lineNo)
		: eventType(_eventType), errorType(_errorType), classIndex(_classIndex), functionIndex(_functionIndex), lineNo(_lineNo), LinkedListItem("", 0, _classIndex)
	{ } // EErrorType constructor
	ErrorRptItem(EEventType _eventType, EErrorType _errorType) : LinkedListItem(),
		eventType(_eventType), errorType(_errorType)
	{ } // EErrorType constructor

	void PrintItem();
	EClass classIndex{ EClass::none };
	uint8_t functionIndex{ 0 };
	uint16_t lineNo{ 0 };
private:
	//static uint16_t classIndex{(uint16_t) EClass::ErrorRptEvent};
	//char** functionLabels;
	EEventType eventType{ EEventType::none };
	EErrorType errorType{ EErrorType::none };
}; // class ErrorRptEvent

class ErrorRptList : public LinkedListManage {
public:
	ErrorRptList(char* _name, uint16_t _ID, EClass _classID)
		: LinkedListManage(_name, _ID, _classID) {}
	void PrintList();
	void AppendItem(ErrorRptItem* _errorRptItem);
private:
#ifdef USE_CLASSNAME
	CandFNames* classNameList;
#endif // USE_CLASSNAME
}; //  class ErrorRptEventMgt

#endif // USE_ERROR_RPT

#endif // ERROR_RPT_H
