#ifndef SAFE_ARRAY_H
#define SAFE_ARRAY_H

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"
#include <setjmp.h>

//#define _DEBUG_SAFE_ARRAY
extern jmp_buf envAbort;

// SafeArray maintains an array of type T pointers and controls indexing into the array 
// by overloading the [] index operator.
template <typename T> class SafeArray {
public:
	// constructor, array size initialization parameter
	SafeArray(uint16_t _size) : arraySize(_size) { 
#ifdef _DEBUG_SAFE_ARRAY
		PRINT_FUNCTION_ENTER_NL;
#endif //  _DEBUG_SAFE_ARRAY
		// initialize each item to 99 for data validation
		for (size_t i = 0; i < arraySize; i++) array[i] = 99;
#ifdef _DEBUG_SAFE_ARRAY
		PRINT_FUNCTION_EXIT_NL;
#endif //  _DEBUG_SAFE_ARRAY
		return;
	} // SafeArray
	~SafeArray() {}; // destructor
	bool operator[] (size_t _index) {// overload the [] operator
		if (_index < 0 || _index >= arraySize) {
			Serial.print("ERROR Index range should be 0 to "); Serial.print(arraySize - 1);
			Serial.print(", current index = "); Serial.println(_index);
			// Do some error handling
#ifdef _DEBUG_SAFE_ARRAY
			PRINT_FUNCTION_EXIT_NL;
#endif //  _DEBUG_SAFE_ARRAY
			return false;
		}
		else {
#ifdef _DEBUG_SAFE_ARRAY
			Serial.print("OK Index range 0 to "); Serial.print(arraySize - 1);
			Serial.print(", current index = "); Serial.println(_index);
			PRINT_FUNCTION_EXIT_NL;
#endif //  _DEBUG_SAFE_ARRAY
			return true;
		}
	} // Operator[]
private:
	T* dummy{ 0 }; // returned if error encountered
	uint16_t arraySize;
public:
	T* array[]; //
}; // class SafeArray 


#endif // SAFE_ARRAY_H
