// Begin Primes.cpp

// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to
// incorporate it inside of an executable or other machine language component
// without restriction for personal or commercial use.The Software may not be
// redistributed as source code or in any recognizable human readable form
// in any form whatsoever for any use whatsoever.
// 
// The Software is the property of Michle Delsol(France), copyright 2023 USA and
// international. For any questions concerning use of the software contact
// Michle Delsol at CPParduino@md-dsl.fr.
// 
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#include "Globals.h"

//#define _DEBUG_PRIMES_LIST_IS_PRIME
//#define _DEBUG_PRIMES_LIST_ADD_PRIME_NO
//#define _DEBUG_PRIMES_LIST_GENERATE_PRIMES


const char* primesErrorsLabels[]{ "no too big", "alloc fail" };
// Scenario

//****************************  PRIME ITEM ************************************

PrimeItem::PrimeItem(uint16_t _primeNumber)
	: primeNumber(_primeNumber), LinkedListItem() {
	//PRINT_FUNCTION_ENTER_NL;
	// Do nothing in constructor
	//PRINT_FUNCTION_EXIT_NL;
} // Primes()

//****************************  PRIME LIST ************************************

PrimesList::PrimesList(uint16_t _limit, char* _listName)
	: limit(_limit), LinkedListManage(_listName) {
	PRINT_FUNCTION_ENTER_NL;
	// Get the list going
	AddPrimeNo(2);
	AddPrimeNo(3);
	PrintPrimes();
	PRINT_FUNCTION_EXIT_NL;
} // PrimesList::PrimesList

uint16_t PrimesList::GetLastPrimeNumber() {
	PrimeItem* lastItem = GetTail()->GetPrevious();
	uint16_t lastPrimeNumber = lastItem->GetPrimeNumber();
	return lastPrimeNumber;
} // PrimesList::GetLastPrimeNumber

void PrimesList::GeneratePrimes() {
#ifdef _DEBUG_PRIMES_LIST_GENERATE_PRIMES
	PRINT_FUNCTION_ENTER_NL;
	PRINT_VALUE_EQUAL_NL(limit);
#endif // _DEBUG_PRIMES_LIST_GENERATE_PRIMES
	for (uint16_t i = 4; i < limit; i++) {
		uint16_t lastPrimeNumber = GetLastPrimeNumber();
		if (sqrt(i) >= lastPrimeNumber ) {
			// Past square root, no need to look further
			break;
		}
		bool isPrime = IsPrime(i);
		if (isPrime == true) {
#ifdef _DEBUG_PRIMES_LIST_GENERATE_PRIMES
			PRINT_VALUE_EQUAL_DESC_NL("GeneratePrimes - Got prime ", i);
#endif // _DEBUG_PRIMES_LIST_GENERATE_PRIMES
			uint8_t* allocSmallQty = (uint8_t * ) malloc(20);
			// 10 - get garbled output
			// 15 - GetAvailableSpace at end does not work
			// 20 - results OK
			if (allocSmallQty == 0) {
				PRINT_VALUE_EQUAL_DESC_NL("Number of primes generated ", i+1);
				Serial.println(F("Getting short on available memory - stopping."));
				free(allocSmallQty = allocSmallQty);
				break;
			}
			free(allocSmallQty = allocSmallQty);
			bool addPrime = AddPrimeNo(i);
			if (addPrime == false) {
#ifdef _DEBUG_PRIMES_LIST_GENERATE_PRIMES
				PRINT_VALUE_EQUAL_DESC_NL("GeneratePrimes - Error on number ", i);
#endif // _DEBUG_PRIMES_LIST_GENERATE_PRIMES
				delay(100);
			} // if (addPrime == false)
		} // if (isPrime == true)
	} // for (uint16_t i = 4; i < limit; i++)
#ifdef _DEBUG_PRIMES_LIST_GENERATE_PRIMES
	PRINT_FUNCTION_EXIT_NL;
#endif // _DEBUG_PRIMES_LIST_GENERATE_PRIMES
	return;
} // PrimesList::GeneratePrimes

bool PrimesList::IsPrime(uint16_t _num) {
#ifdef _DEBUG_PRIMES_LIST_IS_PRIME
	PRINT_FUNCTION_ENTER_NL;
	PRINT_VALUE_EQUAL_DESC_NL("IsPrime - entering with ", _num);
#endif // _DEBUG_PRIMES_LIST_IS_PRIME
	uint16_t num = _num;
	bool numberIsPrime{ true };
	// Check that number does not exceed last prime number
	PrimeItem* lastItem = GetTail()->GetPrevious();
	uint16_t lastPrimeNo = lastItem->GetPrimeNumber();
	// Check number against existing primes
#ifdef _DEBUG_PRIMES_LIST_IS_PRIME
	PRINT_VALUE_EQUAL_TAB_NL((long)GetTail());
	PRINT_VALUE_EQUAL_TAB_NL((long)GetHead());
	PRINT_VALUE_EQUAL_TAB_NL((long)GetHead()->GetNext());
	PRINT_VALUE_EQUAL_TAB_NL((long)GetHead()->GetNext()->GetNext());
	PRINT_VALUE_EQUAL_TAB_NL((long)GetHead()->GetNext()->GetNext()->GetNext());
	PRINT_VALUE_EQUAL_TAB_NL((long)GetHead()->GetNext()->GetNext()->GetNext()->GetNext());
	PRINT_VALUE_EQUAL_TAB_NL((long)GetHead()->GetNext()->GetNext()->GetNext()->GetNext()->GetNext());
#endif // _DEBUG_PRIMES_LIST_IS_PRIME
	for (PrimeItem* primePtr = GetHead()->GetNext(); primePtr != GetTail(); primePtr = primePtr->GetNext()) {
		uint16_t checkAgainst = primePtr->GetPrimeNumber();
		uint16_t currentPrime = primePtr->GetPrimeNumber();
		uint16_t moduloOp = num % currentPrime;
#ifdef _DEBUG_PRIMES_LIST_IS_PRIME
		PRINT_VALUE_EQUAL_NL((long)primePtr);
		PRINT_VALUE_EQUAL_TAB_NL(num);
		PRINT_VALUE_EQUAL_TAB_NL(checkAgainst);
		PRINT_VALUE_EQUAL_TAB_NL(moduloOp);
#endif // _DEBUG_PRIMES_LIST_IS_PRIME
		if (moduloOp == 0) {
			// Not a prime number, return
			numberIsPrime = false;
			break;
		}
	}
#ifdef _DEBUG_PRIMES_LIST_IS_PRIME
	PRINT_FUNCTION_EXIT_NL;
#endif // _DEBUG_PRIMES_LIST_IS_PRIME
	return numberIsPrime;
} // Primes::IsPrime

bool PrimesList::AddPrimeNo(uint16_t _num) {
	// append item to link list of prime numbers
#ifdef _DEBUG_PRIMES_LIST_ADD_PRIME_NO
	PRINT_VALUE_EQUAL_DESC_NL("AddPrimeNo ", _num);
#endif // _DEBUG_PRIMES_LIST_ADD_PRIME_NO
	PrimeItem* primeNo = new PrimeItem(_num);
	if (primeNo == 0) {
		// ERROR - could not allocate for prime.
#ifdef _DEBUG_PRIMES_LIST_ADD_PRIME_NO
		PRINT_VALUE_EQUAL_DESC_NL("AddPrimeNo could not allocate on ", _num);
#endif // _DEBUG_PRIMES_LIST_ADD_PRIME_NO
		longjmp(primesJmp, (uint8_t)EErrors::allocFail);
		//delay(100);
		//return false;
	}
	AddItem(primeNo); // calling LinkedListManage method - takes pointer
	return true;
} // PrimesList::AddPrimeNo

// Function to generate prime numbers up to a given limit 
void PrimesList::PrintPrimes() {
	PRINT_FUNCTION_ENTER_NL;
	uint16_t primesCount{ GetItemCount() };
	PRINT_VALUE_EQUAL_NL(primesCount);
	//PRINT_VALUE_EQUAL_NL((long)GetHead());
	//PRINT_VALUE_EQUAL_NL((long)(GetHead()->GetNext()));
	//PRINT_VALUE_EQUAL_NL((long)GetTail());

	PrimeItem* primePtr = GetHead()->GetNext();
	//for (PrimeItem* primePtr = GetHead()->GetNext(); primePtr != GetTail(); primePtr = primePtr->GetNext()) {
	for (uint16_t i = 0; i < primesCount; i++) {
		PRINT_VALUE_EQUAL(i+1); Serial.print(F(" "));
		PRINT_VALUE_EQUAL((long)primePtr); Serial.print(F(" "));
		PRINT_VALUE_EQUAL_NL(primePtr->GetPrimeNumber()); primePtr = primePtr->GetNext();
	}
	PRINT_FUNCTION_EXIT_NL
} // PrimesList::PrintPrimes

// End ATest.cpp
